import java.io
import java.lang
import java.util.concurrent
import java.util.concurrent.atomic
import scala
import scala.collection
import scala.collection.generic
import scala.collection.immutable
import scala.collection.mutable
import scala.collection.parallel
import scala.collection.parallel.mutable
import scala.math
import scala.reflect
import scala.runtime
import scala.util.hashing
import typing



class BasicNode:
    def __init__(self): ...
    def string(self, int: int) -> java.lang.String: ...

class Debug:
    @staticmethod
    def clear() -> None: ...
    @staticmethod
    def flush() -> None: ...
    @staticmethod
    def log(s: typing.Any) -> bool: ...
    @staticmethod
    def logbuffer() -> java.util.concurrent.ConcurrentLinkedQueue[typing.Any]: ...

_KVNode__K = typing.TypeVar('_KVNode__K')  # <K>
_KVNode__V = typing.TypeVar('_KVNode__V')  # <V>
class KVNode(typing.Generic[_KVNode__K, _KVNode__V]):
    def kvPair(self) -> scala.Tuple2[_KVNode__K, _KVNode__V]: ...

_Map__A = typing.TypeVar('_Map__A')  # <A>
_Map__B = typing.TypeVar('_Map__B')  # <B>
class Map(scala.collection.mutable.Map[_Map__A, _Map__B], typing.Generic[_Map__A, _Map__B]):
    @staticmethod
    def $init$($this: 'Map') -> None: ...
    def equals(self, that: typing.Any) -> bool: ...
    def getOrElseUpdate(self, key: _Map__A, op: scala.Function0[_Map__B]) -> _Map__B: ...
    def hashCode(self) -> int: ...
    def putIfAbsent(self, k: _Map__A, v: _Map__B) -> scala.Option[_Map__B]: ...
    @typing.overload
    def remove(self, k: _Map__A, v: _Map__B) -> bool: ...
    @typing.overload
    def remove(self, key: typing.Any) -> scala.Option[typing.Any]: ...
    @typing.overload
    def replace(self, k: _Map__A, oldvalue: _Map__B, newvalue: _Map__B) -> bool: ...
    @typing.overload
    def replace(self, k: _Map__A, v: _Map__B) -> scala.Option[_Map__B]: ...
    def toString(self) -> java.lang.String: ...

_RDCSS_Descriptor__K = typing.TypeVar('_RDCSS_Descriptor__K')  # <K>
_RDCSS_Descriptor__V = typing.TypeVar('_RDCSS_Descriptor__V')  # <V>
class RDCSS_Descriptor(scala.Product, scala.Serializable, typing.Generic[_RDCSS_Descriptor__K, _RDCSS_Descriptor__V]):
    def __init__(self, old: 'INode'[_RDCSS_Descriptor__K, _RDCSS_Descriptor__V], expectedmain: 'MainNode'[_RDCSS_Descriptor__K, _RDCSS_Descriptor__V], nv: 'INode'[_RDCSS_Descriptor__K, _RDCSS_Descriptor__V]): ...
    _apply__K = typing.TypeVar('_apply__K')  # <K>
    _apply__V = typing.TypeVar('_apply__V')  # <V>
    @staticmethod
    def apply(old: 'INode'[_apply__K, _apply__V], expectedmain: 'MainNode'[_apply__K, _apply__V], nv: 'INode'[_apply__K, _apply__V]) -> 'RDCSS_Descriptor'[_apply__K, _apply__V]: ...
    def canEqual(self, x$1: typing.Any) -> bool: ...
    def committed(self) -> bool: ...
    def committed_$eq(self, x$1: bool) -> None: ...
    _copy__K = typing.TypeVar('_copy__K')  # <K>
    _copy__V = typing.TypeVar('_copy__V')  # <V>
    def copy(self, old: 'INode'[typing.Any, typing.Any], expectedmain: 'MainNode'[typing.Any, typing.Any], nv: 'INode'[typing.Any, typing.Any]) -> 'RDCSS_Descriptor'[typing.Any, typing.Any]: ...
    _copy$default$1__K = typing.TypeVar('_copy$default$1__K')  # <K>
    _copy$default$1__V = typing.TypeVar('_copy$default$1__V')  # <V>
    def copy$default$1(self) -> 'INode'[typing.Any, typing.Any]: ...
    _copy$default$2__K = typing.TypeVar('_copy$default$2__K')  # <K>
    _copy$default$2__V = typing.TypeVar('_copy$default$2__V')  # <V>
    def copy$default$2(self) -> 'MainNode'[typing.Any, typing.Any]: ...
    _copy$default$3__K = typing.TypeVar('_copy$default$3__K')  # <K>
    _copy$default$3__V = typing.TypeVar('_copy$default$3__V')  # <V>
    def copy$default$3(self) -> 'INode'[typing.Any, typing.Any]: ...
    def equals(self, x$1: typing.Any) -> bool: ...
    def expectedmain(self) -> 'MainNode'[_RDCSS_Descriptor__K, _RDCSS_Descriptor__V]: ...
    def hashCode(self) -> int: ...
    def nv(self) -> 'INode'[_RDCSS_Descriptor__K, _RDCSS_Descriptor__V]: ...
    def old(self) -> 'INode'[_RDCSS_Descriptor__K, _RDCSS_Descriptor__V]: ...
    def productArity(self) -> int: ...
    def productElement(self, x$1: int) -> typing.Any: ...
    def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
    def productPrefix(self) -> java.lang.String: ...
    def toString(self) -> java.lang.String: ...
    _unapply__K = typing.TypeVar('_unapply__K')  # <K>
    _unapply__V = typing.TypeVar('_unapply__V')  # <V>
    @staticmethod
    def unapply(x$0: 'RDCSS_Descriptor'[_unapply__K, _unapply__V]) -> scala.Option[scala.Tuple3['INode'[_unapply__K, _unapply__V], 'MainNode'[_unapply__K, _unapply__V], 'INode'[_unapply__K, _unapply__V]]]: ...

class RestartException:
    @staticmethod
    def addSuppressed(x$1: java.lang.Throwable) -> None: ...
    @staticmethod
    def fillInStackTrace() -> java.lang.Throwable: ...
    @staticmethod
    def getCause() -> java.lang.Throwable: ...
    @staticmethod
    def getLocalizedMessage() -> java.lang.String: ...
    @staticmethod
    def getMessage() -> java.lang.String: ...
    @staticmethod
    def getStackTrace() -> typing.List[java.lang.StackTraceElement]: ...
    @staticmethod
    def getSuppressed() -> typing.List[java.lang.Throwable]: ...
    @staticmethod
    def initCause(x$1: java.lang.Throwable) -> java.lang.Throwable: ...
    @typing.overload
    @staticmethod
    def printStackTrace() -> None: ...
    @typing.overload
    @staticmethod
    def printStackTrace(x$1: java.io.PrintStream) -> None: ...
    @typing.overload
    @staticmethod
    def printStackTrace(x$1: java.io.PrintWriter) -> None: ...
    @staticmethod
    def setStackTrace(x$1: typing.List[java.lang.StackTraceElement]) -> None: ...
    @staticmethod
    def toString() -> java.lang.String: ...

_TrieMapIterator__K = typing.TypeVar('_TrieMapIterator__K')  # <K>
_TrieMapIterator__V = typing.TypeVar('_TrieMapIterator__V')  # <V>
class TrieMapIterator(scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]], typing.Generic[_TrieMapIterator__K, _TrieMapIterator__V]):
    def __init__(self, level: int, ct: 'TrieMap'[_TrieMapIterator__K, _TrieMapIterator__V], mustInit: bool): ...
    _$colon$bslash__B = typing.TypeVar('_$colon$bslash__B')  # <B>
    def $colon$bslash(self, z: _.colon.bslash__B, op: scala.Function2[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _.colon.bslash__B, _.colon.bslash__B]) -> _.colon.bslash__B: ...
    _$div$colon__B = typing.TypeVar('_$div$colon__B')  # <B>
    def $div$colon(self, z: _.div.colon__B, op: scala.Function2[_.div.colon__B, scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _.div.colon__B]) -> _.div.colon__B: ...
    _$lessinit$greater$default$3__K = typing.TypeVar('_$lessinit$greater$default$3__K')  # <K>
    _$lessinit$greater$default$3__V = typing.TypeVar('_$lessinit$greater$default$3__V')  # <V>
    @staticmethod
    def $lessinit$greater$default$3() -> bool: ...
    _$plus$plus__B = typing.TypeVar('_$plus$plus__B')  # <B>
    def $plus$plus(self, that: scala.Function0[scala.collection.GenTraversableOnce[_.plus.plus__B]]) -> scala.collection.Iterator[_.plus.plus__B]: ...
    @typing.overload
    def addString(self, b: scala.collection.mutable.StringBuilder) -> scala.collection.mutable.StringBuilder: ...
    @typing.overload
    def addString(self, b: scala.collection.mutable.StringBuilder, sep: typing.Union[java.lang.String, str]) -> scala.collection.mutable.StringBuilder: ...
    @typing.overload
    def addString(self, b: scala.collection.mutable.StringBuilder, start: typing.Union[java.lang.String, str], sep: typing.Union[java.lang.String, str], end: typing.Union[java.lang.String, str]) -> scala.collection.mutable.StringBuilder: ...
    def advance(self) -> None: ...
    _aggregate__B = typing.TypeVar('_aggregate__B')  # <B>
    def aggregate(self, z: scala.Function0[_aggregate__B], seqop: scala.Function2[_aggregate__B, scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _aggregate__B], combop: scala.Function2[_aggregate__B, _aggregate__B, _aggregate__B]) -> _aggregate__B: ...
    def buffered(self) -> scala.collection.BufferedIterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    _collect__B = typing.TypeVar('_collect__B')  # <B>
    def collect(self, pf: scala.PartialFunction[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _collect__B]) -> scala.collection.Iterator[_collect__B]: ...
    _collectFirst__B = typing.TypeVar('_collectFirst__B')  # <B>
    def collectFirst(self, pf: scala.PartialFunction[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _collectFirst__B]) -> scala.Option[_collectFirst__B]: ...
    def contains(self, elem: typing.Any) -> bool: ...
    _copyToArray_0__B = typing.TypeVar('_copyToArray_0__B')  # <B>
    _copyToArray_1__B = typing.TypeVar('_copyToArray_1__B')  # <B>
    _copyToArray_2__B = typing.TypeVar('_copyToArray_2__B')  # <B>
    @typing.overload
    def copyToArray(self, xs: typing.Any) -> None: ...
    @typing.overload
    def copyToArray(self, xs: typing.Any, start: int) -> None: ...
    @typing.overload
    def copyToArray(self, xs: typing.Any, start: int, len: int) -> None: ...
    _copyToBuffer__B = typing.TypeVar('_copyToBuffer__B')  # <B>
    def copyToBuffer(self, dest: scala.collection.mutable.Buffer[_copyToBuffer__B]) -> None: ...
    _corresponds__B = typing.TypeVar('_corresponds__B')  # <B>
    def corresponds(self, that: scala.collection.GenTraversableOnce[_corresponds__B], p: scala.Function2[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _corresponds__B, typing.Any]) -> bool: ...
    def count(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> int: ...
    def drop(self, n: int) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def dropWhile(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def dupTo(self, it: 'TrieMapIterator'[_TrieMapIterator__K, _TrieMapIterator__V]) -> None: ...
    def duplicate(self) -> scala.Tuple2[scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]], scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]]: ...
    def exists(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> bool: ...
    def filter(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def filterNot(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def find(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> scala.Option[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    _flatMap__B = typing.TypeVar('_flatMap__B')  # <B>
    def flatMap(self, f: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], scala.collection.GenTraversableOnce[_flatMap__B]]) -> scala.collection.Iterator[_flatMap__B]: ...
    _fold__A1 = typing.TypeVar('_fold__A1')  # <A1>
    def fold(self, z: _fold__A1, op: scala.Function2[_fold__A1, _fold__A1, _fold__A1]) -> _fold__A1: ...
    _foldLeft__B = typing.TypeVar('_foldLeft__B')  # <B>
    def foldLeft(self, z: _foldLeft__B, op: scala.Function2[_foldLeft__B, scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _foldLeft__B]) -> _foldLeft__B: ...
    _foldRight__B = typing.TypeVar('_foldRight__B')  # <B>
    def foldRight(self, z: _foldRight__B, op: scala.Function2[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _foldRight__B, _foldRight__B]) -> _foldRight__B: ...
    def forall(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> bool: ...
    _foreach__U = typing.TypeVar('_foreach__U')  # <U>
    def foreach(self, f: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _foreach__U]) -> None: ...
    _grouped__B = typing.TypeVar('_grouped__B')  # <B>
    def grouped(self, size: int) -> scala.collection.Iterator.GroupedIterator[_grouped__B]: ...
    def hasDefiniteSize(self) -> bool: ...
    def hasNext(self) -> bool: ...
    _indexOf_0__B = typing.TypeVar('_indexOf_0__B')  # <B>
    _indexOf_1__B = typing.TypeVar('_indexOf_1__B')  # <B>
    @typing.overload
    def indexOf(self, elem: _indexOf_0__B) -> int: ...
    @typing.overload
    def indexOf(self, elem: _indexOf_1__B, from_: int) -> int: ...
    @typing.overload
    def indexWhere(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> int: ...
    @typing.overload
    def indexWhere(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any], from_: int) -> int: ...
    def isEmpty(self) -> bool: ...
    def isTraversableAgain(self) -> bool: ...
    def length(self) -> int: ...
    def level(self) -> int: ...
    def level_$eq(self, x$1: int) -> None: ...
    _map__B = typing.TypeVar('_map__B')  # <B>
    def map(self, f: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _map__B]) -> scala.collection.Iterator[_map__B]: ...
    def max(self, cmp: scala.math.Ordering) -> typing.Any: ...
    def maxBy(self, f: scala.Function1, cmp: scala.math.Ordering) -> typing.Any: ...
    def min(self, cmp: scala.math.Ordering) -> typing.Any: ...
    def minBy(self, f: scala.Function1, cmp: scala.math.Ordering) -> typing.Any: ...
    @typing.overload
    def mkString(self) -> java.lang.String: ...
    @typing.overload
    def mkString(self, sep: typing.Union[java.lang.String, str]) -> java.lang.String: ...
    @typing.overload
    def mkString(self, start: typing.Union[java.lang.String, str], sep: typing.Union[java.lang.String, str], end: typing.Union[java.lang.String, str]) -> java.lang.String: ...
    def newIterator(self, _lev: int, _ct: 'TrieMap'[_TrieMapIterator__K, _TrieMapIterator__V], _mustInit: bool) -> 'TrieMapIterator'[_TrieMapIterator__K, _TrieMapIterator__V]: ...
    def next(self) -> scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]: ...
    def nonEmpty(self) -> bool: ...
    _padTo__A1 = typing.TypeVar('_padTo__A1')  # <A1>
    def padTo(self, len: int, elem: _padTo__A1) -> scala.collection.Iterator[_padTo__A1]: ...
    def partition(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> scala.Tuple2[scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]], scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]]: ...
    _patch__B = typing.TypeVar('_patch__B')  # <B>
    def patch(self, from_: int, patchElems: scala.collection.Iterator[_patch__B], replaced: int) -> scala.collection.Iterator[_patch__B]: ...
    def printDebug(self) -> None: ...
    _product__B = typing.TypeVar('_product__B')  # <B>
    def product(self, num: scala.math.Numeric[_product__B]) -> _product__B: ...
    _reduce__A1 = typing.TypeVar('_reduce__A1')  # <A1>
    def reduce(self, op: scala.Function2[_reduce__A1, _reduce__A1, _reduce__A1]) -> _reduce__A1: ...
    _reduceLeft__B = typing.TypeVar('_reduceLeft__B')  # <B>
    def reduceLeft(self, op: scala.Function2[_reduceLeft__B, scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _reduceLeft__B]) -> _reduceLeft__B: ...
    _reduceLeftOption__B = typing.TypeVar('_reduceLeftOption__B')  # <B>
    def reduceLeftOption(self, op: scala.Function2[_reduceLeftOption__B, scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _reduceLeftOption__B]) -> scala.Option[_reduceLeftOption__B]: ...
    _reduceOption__A1 = typing.TypeVar('_reduceOption__A1')  # <A1>
    def reduceOption(self, op: scala.Function2[_reduceOption__A1, _reduceOption__A1, _reduceOption__A1]) -> scala.Option[_reduceOption__A1]: ...
    _reduceRight__B = typing.TypeVar('_reduceRight__B')  # <B>
    def reduceRight(self, op: scala.Function2[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _reduceRight__B, _reduceRight__B]) -> _reduceRight__B: ...
    _reduceRightOption__B = typing.TypeVar('_reduceRightOption__B')  # <B>
    def reduceRightOption(self, op: scala.Function2[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _reduceRightOption__B, _reduceRightOption__B]) -> scala.Option[_reduceRightOption__B]: ...
    def reversed(self) -> scala.collection.immutable.List[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def sameElements(self, that: scala.collection.Iterator[typing.Any]) -> bool: ...
    _scanLeft__B = typing.TypeVar('_scanLeft__B')  # <B>
    def scanLeft(self, z: _scanLeft__B, op: scala.Function2[_scanLeft__B, scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _scanLeft__B]) -> scala.collection.Iterator[_scanLeft__B]: ...
    _scanRight__B = typing.TypeVar('_scanRight__B')  # <B>
    def scanRight(self, z: _scanRight__B, op: scala.Function2[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _scanRight__B, _scanRight__B]) -> scala.collection.Iterator[_scanRight__B]: ...
    def seq(self) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def size(self) -> int: ...
    def sizeHintIfCheap(self) -> int: ...
    def slice(self, from_: int, until: int) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def sliceIterator(self, from_: int, until: int) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    _sliding__B = typing.TypeVar('_sliding__B')  # <B>
    def sliding(self, size: int, step: int) -> scala.collection.Iterator.GroupedIterator[_sliding__B]: ...
    _sliding$default$2__B = typing.TypeVar('_sliding$default$2__B')  # <B>
    def sliding$default$2(self) -> int: ...
    def span(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> scala.Tuple2[scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]], scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]]: ...
    def subdivide(self) -> scala.collection.Seq[scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]]: ...
    _sum__B = typing.TypeVar('_sum__B')  # <B>
    def sum(self, num: scala.math.Numeric[_sum__B]) -> _sum__B: ...
    def take(self, n: int) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def takeWhile(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    _to__Col = typing.TypeVar('_to__Col')  # <Col>
    def to(self, cbf: scala.collection.generic.CanBuildFrom[scala.runtime.Nothing., scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _to__Col]) -> _to__Col: ...
    _toArray__B = typing.TypeVar('_toArray__B')  # <B>
    def toArray(self, evidence$1: scala.reflect.ClassTag[_toArray__B]) -> typing.Any: ...
    _toBuffer__B = typing.TypeVar('_toBuffer__B')  # <B>
    def toBuffer(self) -> scala.collection.mutable.Buffer[_toBuffer__B]: ...
    def toIndexedSeq(self) -> scala.collection.immutable.IndexedSeq[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def toIterable(self) -> scala.collection.Iterable[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def toIterator(self) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def toList(self) -> scala.collection.immutable.List[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    _toMap__T = typing.TypeVar('_toMap__T')  # <T>
    _toMap__U = typing.TypeVar('_toMap__U')  # <U>
    def toMap(self, ev: scala.Predef..less.colon.less[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], scala.Tuple2[_toMap__T, _toMap__U]]) -> scala.collection.immutable.Map[_toMap__T, _toMap__U]: ...
    def toSeq(self) -> scala.collection.Seq[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    _toSet__B = typing.TypeVar('_toSet__B')  # <B>
    def toSet(self) -> scala.collection.immutable.Set[_toSet__B]: ...
    def toStream(self) -> scala.collection.immutable.Stream[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def toString(self) -> java.lang.String: ...
    def toTraversable(self) -> scala.collection.Traversable[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def toVector(self) -> scala.collection.immutable.Vector[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    def withFilter(self, p: scala.Function1[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]) -> scala.collection.Iterator[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V]]: ...
    _zip__B = typing.TypeVar('_zip__B')  # <B>
    def zip(self, that: scala.collection.Iterator[_zip__B]) -> scala.collection.Iterator[scala.Tuple2[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], _zip__B]]: ...
    _zipAll__B = typing.TypeVar('_zipAll__B')  # <B>
    _zipAll__A1 = typing.TypeVar('_zipAll__A1')  # <A1>
    _zipAll__B1 = typing.TypeVar('_zipAll__B1')  # <B1>
    def zipAll(self, that: scala.collection.Iterator[_zipAll__B], thisElem: _zipAll__A1, thatElem: _zipAll__B1) -> scala.collection.Iterator[scala.Tuple2[_zipAll__A1, _zipAll__B1]]: ...
    def zipWithIndex(self) -> scala.collection.Iterator[scala.Tuple2[scala.Tuple2[_TrieMapIterator__K, _TrieMapIterator__V], typing.Any]]: ...

class TrieMapSerializationEnd:
    @staticmethod
    def canEqual(x$1: typing.Any) -> bool: ...
    @staticmethod
    def hashCode() -> int: ...
    @staticmethod
    def productArity() -> int: ...
    @staticmethod
    def productElement(x$1: int) -> typing.Any: ...
    @staticmethod
    def productIterator() -> scala.collection.Iterator[typing.Any]: ...
    @staticmethod
    def productPrefix() -> java.lang.String: ...
    @staticmethod
    def toString() -> java.lang.String: ...

class MainNode: ...

_FailedNode__K = typing.TypeVar('_FailedNode__K')  # <K>
_FailedNode__V = typing.TypeVar('_FailedNode__V')  # <V>
class FailedNode(MainNode[_FailedNode__K, _FailedNode__V], typing.Generic[_FailedNode__K, _FailedNode__V]):
    def __init__(self, p: MainNode[_FailedNode__K, _FailedNode__V]): ...
    def cachedSize(self, ct: typing.Any) -> int: ...
    def string(self, lev: int) -> scala.runtime.Nothing.: ...
    def toString(self) -> java.lang.String: ...

_LNode__K = typing.TypeVar('_LNode__K')  # <K>
_LNode__V = typing.TypeVar('_LNode__V')  # <V>
class LNode(MainNode[_LNode__K, _LNode__V], typing.Generic[_LNode__K, _LNode__V]):
    @typing.overload
    def __init__(self, k: _LNode__K, v: _LNode__V): ...
    @typing.overload
    def __init__(self, k1: _LNode__K, v1: _LNode__V, k2: _LNode__K, v2: _LNode__V): ...
    @typing.overload
    def __init__(self, listmap: scala.collection.immutable.ListMap[_LNode__K, _LNode__V]): ...
    def cachedSize(self, ct: typing.Any) -> int: ...
    def get(self, k: _LNode__K) -> scala.Option[_LNode__V]: ...
    def inserted(self, k: _LNode__K, v: _LNode__V) -> 'LNode'[_LNode__K, _LNode__V]: ...
    def listmap(self) -> scala.collection.immutable.ListMap[_LNode__K, _LNode__V]: ...
    def removed(self, k: _LNode__K, ct: 'TrieMap'[_LNode__K, _LNode__V]) -> MainNode[_LNode__K, _LNode__V]: ...
    def string(self, lev: int) -> java.lang.String: ...

_SNode__K = typing.TypeVar('_SNode__K')  # <K>
_SNode__V = typing.TypeVar('_SNode__V')  # <V>
class SNode(BasicNode, KVNode[_SNode__K, _SNode__V], typing.Generic[_SNode__K, _SNode__V]):
    def __init__(self, k: _SNode__K, v: _SNode__V, hc: int): ...
    def copy(self) -> 'SNode'[_SNode__K, _SNode__V]: ...
    def copyTombed(self) -> 'TNode'[_SNode__K, _SNode__V]: ...
    def copyUntombed(self) -> 'SNode'[_SNode__K, _SNode__V]: ...
    def hc(self) -> int: ...
    def k(self) -> _SNode__K: ...
    def kvPair(self) -> scala.Tuple2[_SNode__K, _SNode__V]: ...
    def string(self, lev: int) -> java.lang.String: ...
    def v(self) -> _SNode__V: ...

_TNode__K = typing.TypeVar('_TNode__K')  # <K>
_TNode__V = typing.TypeVar('_TNode__V')  # <V>
class TNode(MainNode[_TNode__K, _TNode__V], KVNode[_TNode__K, _TNode__V], typing.Generic[_TNode__K, _TNode__V]):
    def __init__(self, k: _TNode__K, v: _TNode__V, hc: int): ...
    def cachedSize(self, ct: typing.Any) -> int: ...
    def copy(self) -> 'TNode'[_TNode__K, _TNode__V]: ...
    def copyTombed(self) -> 'TNode'[_TNode__K, _TNode__V]: ...
    def copyUntombed(self) -> SNode[_TNode__K, _TNode__V]: ...
    def hc(self) -> int: ...
    def k(self) -> _TNode__K: ...
    def kvPair(self) -> scala.Tuple2[_TNode__K, _TNode__V]: ...
    def string(self, lev: int) -> java.lang.String: ...
    def v(self) -> _TNode__V: ...

_TrieMap__MangledHashing__K = typing.TypeVar('_TrieMap__MangledHashing__K')  # <K>
_TrieMap__K = typing.TypeVar('_TrieMap__K')  # <K>
_TrieMap__V = typing.TypeVar('_TrieMap__V')  # <V>
class TrieMap(Map[_TrieMap__K, _TrieMap__V], scala.collection.CustomParallelizable[scala.Tuple2[_TrieMap__K, _TrieMap__V], scala.collection.parallel.mutable.ParTrieMap[_TrieMap__K, _TrieMap__V]], scala.Serializable, typing.Generic[_TrieMap__K, _TrieMap__V]):
    serialVersionUID: typing.ClassVar[int] = ...
    @typing.overload
    def __init__(self): ...
    @typing.overload
    def __init__(self, hashf: scala.util.hashing.Hashing[_TrieMap__K], ef: scala.math.Equiv[_TrieMap__K]): ...
    _$colon$bslash__B = typing.TypeVar('_$colon$bslash__B')  # <B>
    def $colon$bslash(self, z: _.colon.bslash__B, op: scala.Function2[scala.Tuple2[_TrieMap__K, _TrieMap__V], _.colon.bslash__B, _.colon.bslash__B]) -> _.colon.bslash__B: ...
    _$div$colon__B = typing.TypeVar('_$div$colon__B')  # <B>
    def $div$colon(self, z: _.div.colon__B, op: scala.Function2[_.div.colon__B, scala.Tuple2[_TrieMap__K, _TrieMap__V], _.div.colon__B]) -> _.div.colon__B: ...
    @typing.overload
    def $minus(self, key: typing.Any) -> scala.collection.mutable.Map: ...
    @typing.overload
    def $minus(self, elem1: typing.Any, elem2: typing.Any, elems: scala.collection.Seq) -> scala.collection.mutable.Map: ...
    @typing.overload
    def $minus$eq(self, k: _TrieMap__K) -> 'TrieMap'[_TrieMap__K, _TrieMap__V]: ...
    @typing.overload
    def $minus$eq(self, elem1: _TrieMap__K, elem2: _TrieMap__K, elems: scala.collection.Seq[_TrieMap__K]) -> scala.collection.generic.Shrinkable[_TrieMap__K]: ...
    def $minus$minus(self, xs: scala.collection.GenTraversableOnce) -> scala.collection.mutable.Map: ...
    def $minus$minus$eq(self, xs: scala.collection.TraversableOnce[_TrieMap__K]) -> scala.collection.generic.Shrinkable[_TrieMap__K]: ...
    _$plus_0__V1 = typing.TypeVar('_$plus_0__V1')  # <V1>
    _$plus_1__V1 = typing.TypeVar('_$plus_1__V1')  # <V1>
    @typing.overload
    def $plus(self, kv: scala.Tuple2[_TrieMap__K, _.plus_0__V1]) -> scala.collection.mutable.Map[_TrieMap__K, _.plus_0__V1]: ...
    @typing.overload
    def $plus(self, elem1: scala.Tuple2[_TrieMap__K, _.plus_1__V1], elem2: scala.Tuple2[_TrieMap__K, _.plus_1__V1], elems: scala.collection.Seq[scala.Tuple2[_TrieMap__K, _.plus_1__V1]]) -> scala.collection.mutable.Map[_TrieMap__K, _.plus_1__V1]: ...
    @typing.overload
    def $plus$eq(self, kv: scala.Tuple2[_TrieMap__K, _TrieMap__V]) -> 'TrieMap'[_TrieMap__K, _TrieMap__V]: ...
    @typing.overload
    def $plus$eq(self, elem1: typing.Any, elem2: typing.Any, elems: scala.collection.Seq) -> scala.collection.generic.Growable: ...
    _$plus$plus_0__B = typing.TypeVar('_$plus$plus_0__B')  # <B>
    _$plus$plus_0__That = typing.TypeVar('_$plus$plus_0__That')  # <That>
    _$plus$plus_1__V1 = typing.TypeVar('_$plus$plus_1__V1')  # <V1>
    @typing.overload
    def $plus$plus(self, that: scala.collection.GenTraversableOnce[_.plus.plus_0__B], bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], _.plus.plus_0__B, _.plus.plus_0__That]) -> _.plus.plus_0__That: ...
    @typing.overload
    def $plus$plus(self, xs: scala.collection.GenTraversableOnce[scala.Tuple2[_TrieMap__K, _.plus.plus_1__V1]]) -> scala.collection.mutable.Map[_TrieMap__K, _.plus.plus_1__V1]: ...
    _$plus$plus$colon_0__B = typing.TypeVar('_$plus$plus$colon_0__B')  # <B>
    _$plus$plus$colon_0__That = typing.TypeVar('_$plus$plus$colon_0__That')  # <That>
    _$plus$plus$colon_1__B = typing.TypeVar('_$plus$plus$colon_1__B')  # <B>
    _$plus$plus$colon_1__That = typing.TypeVar('_$plus$plus$colon_1__That')  # <That>
    @typing.overload
    def $plus$plus$colon(self, that: scala.collection.Traversable[_.plus.plus.colon_0__B], bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], _.plus.plus.colon_0__B, _.plus.plus.colon_0__That]) -> _.plus.plus.colon_0__That: ...
    @typing.overload
    def $plus$plus$colon(self, that: scala.collection.TraversableOnce[_.plus.plus.colon_1__B], bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], _.plus.plus.colon_1__B, _.plus.plus.colon_1__That]) -> _.plus.plus.colon_1__That: ...
    def $plus$plus$eq(self, xs: scala.collection.TraversableOnce[scala.Tuple2[_TrieMap__K, _TrieMap__V]]) -> scala.collection.generic.Growable[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def CAS_ROOT(self, ov: typing.Any, nv: typing.Any) -> bool: ...
    def RDCSS_READ_ROOT(self, abort: bool) -> 'INode'[_TrieMap__K, _TrieMap__V]: ...
    def RDCSS_READ_ROOT$default$1(self) -> bool: ...
    @typing.overload
    def addString(self, b: scala.collection.mutable.StringBuilder) -> scala.collection.mutable.StringBuilder: ...
    @typing.overload
    def addString(self, b: scala.collection.mutable.StringBuilder, sep: typing.Union[java.lang.String, str]) -> scala.collection.mutable.StringBuilder: ...
    @typing.overload
    def addString(self, b: scala.collection.mutable.StringBuilder, start: typing.Union[java.lang.String, str], sep: typing.Union[java.lang.String, str], end: typing.Union[java.lang.String, str]) -> scala.collection.mutable.StringBuilder: ...
    _aggregate__B = typing.TypeVar('_aggregate__B')  # <B>
    def aggregate(self, z: scala.Function0[_aggregate__B], seqop: scala.Function2[_aggregate__B, scala.Tuple2[_TrieMap__K, _TrieMap__V], _aggregate__B], combop: scala.Function2[_aggregate__B, _aggregate__B, _aggregate__B]) -> _aggregate__B: ...
    _andThen__C = typing.TypeVar('_andThen__C')  # <C>
    def andThen(self, k: scala.Function1[_TrieMap__V, _andThen__C]) -> scala.PartialFunction[_TrieMap__K, _andThen__C]: ...
    def apply(self, k: _TrieMap__K) -> _TrieMap__V: ...
    def apply$mcDD$sp(self, v1: float) -> float: ...
    def apply$mcDF$sp(self, v1: float) -> float: ...
    def apply$mcDI$sp(self, v1: int) -> float: ...
    def apply$mcDJ$sp(self, v1: int) -> float: ...
    def apply$mcFD$sp(self, v1: float) -> float: ...
    def apply$mcFF$sp(self, v1: float) -> float: ...
    def apply$mcFI$sp(self, v1: int) -> float: ...
    def apply$mcFJ$sp(self, v1: int) -> float: ...
    def apply$mcID$sp(self, v1: float) -> int: ...
    def apply$mcIF$sp(self, v1: float) -> int: ...
    def apply$mcII$sp(self, v1: int) -> int: ...
    def apply$mcIJ$sp(self, v1: int) -> int: ...
    def apply$mcJD$sp(self, v1: float) -> int: ...
    def apply$mcJF$sp(self, v1: float) -> int: ...
    def apply$mcJI$sp(self, v1: int) -> int: ...
    def apply$mcJJ$sp(self, v1: int) -> int: ...
    def apply$mcVD$sp(self, v1: float) -> None: ...
    def apply$mcVF$sp(self, v1: float) -> None: ...
    def apply$mcVI$sp(self, v1: int) -> None: ...
    def apply$mcVJ$sp(self, v1: int) -> None: ...
    def apply$mcZD$sp(self, v1: float) -> bool: ...
    def apply$mcZF$sp(self, v1: float) -> bool: ...
    def apply$mcZI$sp(self, v1: int) -> bool: ...
    def apply$mcZJ$sp(self, v1: int) -> bool: ...
    _applyOrElse__K1 = typing.TypeVar('_applyOrElse__K1')  # <K1>
    _applyOrElse__V1 = typing.TypeVar('_applyOrElse__V1')  # <V1>
    def applyOrElse(self, x: _applyOrElse__K1, default: scala.Function1[_applyOrElse__K1, _applyOrElse__V1]) -> _applyOrElse__V1: ...
    _canBuildFrom__K = typing.TypeVar('_canBuildFrom__K')  # <K>
    _canBuildFrom__V = typing.TypeVar('_canBuildFrom__V')  # <V>
    @staticmethod
    def canBuildFrom() -> scala.collection.generic.CanBuildFrom['TrieMap'[typing.Any, typing.Any], scala.Tuple2[_canBuildFrom__K, _canBuildFrom__V], 'TrieMap'[_canBuildFrom__K, _canBuildFrom__V]]: ...
    def canEqual(self, that: typing.Any) -> bool: ...
    def clear(self) -> None: ...
    def clone(self) -> scala.collection.mutable.Map: ...
    _collect__B = typing.TypeVar('_collect__B')  # <B>
    _collect__That = typing.TypeVar('_collect__That')  # <That>
    def collect(self, pf: scala.PartialFunction[scala.Tuple2[_TrieMap__K, _TrieMap__V], _collect__B], bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], _collect__B, _collect__That]) -> _collect__That: ...
    _collectFirst__B = typing.TypeVar('_collectFirst__B')  # <B>
    def collectFirst(self, pf: scala.PartialFunction[scala.Tuple2[_TrieMap__K, _TrieMap__V], _collectFirst__B]) -> scala.Option[_collectFirst__B]: ...
    def companion(self) -> scala.collection.generic.GenericCompanion[scala.collection.mutable.Iterable]: ...
    _compose__A = typing.TypeVar('_compose__A')  # <A>
    def compose(self, g: scala.Function1[_compose__A, _TrieMap__K]) -> scala.Function1[_compose__A, _TrieMap__V]: ...
    def computeHash(self, k: _TrieMap__K) -> int: ...
    def contains(self, key: _TrieMap__K) -> bool: ...
    _copyToArray_0__B = typing.TypeVar('_copyToArray_0__B')  # <B>
    _copyToArray_1__B = typing.TypeVar('_copyToArray_1__B')  # <B>
    _copyToArray_2__B = typing.TypeVar('_copyToArray_2__B')  # <B>
    @typing.overload
    def copyToArray(self, xs: typing.Any) -> None: ...
    @typing.overload
    def copyToArray(self, xs: typing.Any, start: int) -> None: ...
    @typing.overload
    def copyToArray(self, xs: typing.Any, start: int, len: int) -> None: ...
    _copyToBuffer__B = typing.TypeVar('_copyToBuffer__B')  # <B>
    def copyToBuffer(self, dest: scala.collection.mutable.Buffer[_copyToBuffer__B]) -> None: ...
    def count(self, p: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], typing.Any]) -> int: ...
    def default(self, key: _TrieMap__K) -> _TrieMap__V: ...
    def drop(self, n: int) -> typing.Any: ...
    def dropRight(self, n: int) -> typing.Any: ...
    def dropWhile(self, p: scala.Function1) -> typing.Any: ...
    def empty(self) -> 'TrieMap'[_TrieMap__K, _TrieMap__V]: ...
    def equality(self) -> scala.math.Equiv[_TrieMap__K]: ...
    def equals(self, that: typing.Any) -> bool: ...
    def exists(self, p: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], typing.Any]) -> bool: ...
    def filter(self, p: scala.Function1) -> typing.Any: ...
    def filterImpl(self, p: scala.Function1, isFlipped: bool) -> typing.Any: ...
    def filterKeys(self, p: scala.Function1[_TrieMap__K, typing.Any]) -> scala.collection.Map[_TrieMap__K, _TrieMap__V]: ...
    def filterNot(self, p: scala.Function1) -> scala.collection.Map: ...
    def find(self, p: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], typing.Any]) -> scala.Option[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    _flatMap__B = typing.TypeVar('_flatMap__B')  # <B>
    _flatMap__That = typing.TypeVar('_flatMap__That')  # <That>
    def flatMap(self, f: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], scala.collection.GenTraversableOnce[_flatMap__B]], bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], _flatMap__B, _flatMap__That]) -> _flatMap__That: ...
    def flatten(self, asTraversable: scala.Function1) -> scala.collection.GenTraversable: ...
    _fold__A1 = typing.TypeVar('_fold__A1')  # <A1>
    def fold(self, z: _fold__A1, op: scala.Function2[_fold__A1, _fold__A1, _fold__A1]) -> _fold__A1: ...
    _foldLeft__B = typing.TypeVar('_foldLeft__B')  # <B>
    def foldLeft(self, z: _foldLeft__B, op: scala.Function2[_foldLeft__B, scala.Tuple2[_TrieMap__K, _TrieMap__V], _foldLeft__B]) -> _foldLeft__B: ...
    _foldRight__B = typing.TypeVar('_foldRight__B')  # <B>
    def foldRight(self, z: _foldRight__B, op: scala.Function2[scala.Tuple2[_TrieMap__K, _TrieMap__V], _foldRight__B, _foldRight__B]) -> _foldRight__B: ...
    def forall(self, p: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], typing.Any]) -> bool: ...
    _foreach__U = typing.TypeVar('_foreach__U')  # <U>
    def foreach(self, f: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], _foreach__U]) -> None: ...
    _genericBuilder__B = typing.TypeVar('_genericBuilder__B')  # <B>
    def genericBuilder(self) -> scala.collection.mutable.Builder[_genericBuilder__B, scala.collection.mutable.Iterable[_genericBuilder__B]]: ...
    def get(self, k: _TrieMap__K) -> scala.Option[_TrieMap__V]: ...
    _getOrElse__V1 = typing.TypeVar('_getOrElse__V1')  # <V1>
    def getOrElse(self, key: _TrieMap__K, default: scala.Function0[_getOrElse__V1]) -> _getOrElse__V1: ...
    def getOrElseUpdate(self, k: _TrieMap__K, op: scala.Function0[_TrieMap__V]) -> _TrieMap__V: ...
    _groupBy__K = typing.TypeVar('_groupBy__K')  # <K>
    def groupBy(self, f: scala.Function1[scala.Tuple2[typing.Any, _TrieMap__V], typing.Any]) -> scala.collection.immutable.Map[typing.Any, 'TrieMap'[typing.Any, _TrieMap__V]]: ...
    def grouped(self, size: int) -> scala.collection.Iterator['TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    def hasDefiniteSize(self) -> bool: ...
    def hashCode(self) -> int: ...
    def hashing(self) -> scala.util.hashing.Hashing[_TrieMap__K]: ...
    def head(self) -> typing.Any: ...
    def headOption(self) -> scala.Option[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def init(self) -> typing.Any: ...
    def inits(self) -> scala.collection.Iterator['TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    @staticmethod
    def inodeupdater() -> java.util.concurrent.atomic.AtomicReferenceFieldUpdater['INodeBase'[typing.Any, typing.Any], MainNode[typing.Any, typing.Any]]: ...
    def isDefinedAt(self, key: _TrieMap__K) -> bool: ...
    def isEmpty(self) -> bool: ...
    def isReadOnly(self) -> bool: ...
    def isTraversableAgain(self) -> bool: ...
    def iterator(self) -> scala.collection.Iterator[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def keySet(self) -> scala.collection.Set[_TrieMap__K]: ...
    def keys(self) -> scala.collection.Iterable[_TrieMap__K]: ...
    def keysIterator(self) -> scala.collection.Iterator[_TrieMap__K]: ...
    def last(self) -> typing.Any: ...
    def lastOption(self) -> scala.Option[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def lift(self) -> scala.Function1[_TrieMap__K, scala.Option[_TrieMap__V]]: ...
    def lookup(self, k: _TrieMap__K) -> _TrieMap__V: ...
    _map__B = typing.TypeVar('_map__B')  # <B>
    _map__That = typing.TypeVar('_map__That')  # <That>
    def map(self, f: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], _map__B], bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], _map__B, _map__That]) -> _map__That: ...
    _mapResult__NewTo = typing.TypeVar('_mapResult__NewTo')  # <NewTo>
    def mapResult(self, f: scala.Function1['TrieMap'[_TrieMap__K, _TrieMap__V], _mapResult__NewTo]) -> scala.collection.mutable.Builder[scala.Tuple2[_TrieMap__K, _TrieMap__V], _mapResult__NewTo]: ...
    _mapValues__W = typing.TypeVar('_mapValues__W')  # <W>
    def mapValues(self, f: scala.Function1[_TrieMap__V, _mapValues__W]) -> scala.collection.Map[_TrieMap__K, _mapValues__W]: ...
    def max(self, cmp: scala.math.Ordering) -> typing.Any: ...
    def maxBy(self, f: scala.Function1, cmp: scala.math.Ordering) -> typing.Any: ...
    def min(self, cmp: scala.math.Ordering) -> typing.Any: ...
    def minBy(self, f: scala.Function1, cmp: scala.math.Ordering) -> typing.Any: ...
    @typing.overload
    def mkString(self) -> java.lang.String: ...
    @typing.overload
    def mkString(self, sep: typing.Union[java.lang.String, str]) -> java.lang.String: ...
    @typing.overload
    def mkString(self, start: typing.Union[java.lang.String, str], sep: typing.Union[java.lang.String, str], end: typing.Union[java.lang.String, str]) -> java.lang.String: ...
    def newBuilder(self) -> scala.collection.mutable.Builder[scala.Tuple2[_TrieMap__K, _TrieMap__V], 'TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    def nonEmpty(self) -> bool: ...
    def nonReadOnly(self) -> bool: ...
    _orElse__A1 = typing.TypeVar('_orElse__A1')  # <A1>
    _orElse__B1 = typing.TypeVar('_orElse__B1')  # <B1>
    def orElse(self, that: scala.PartialFunction[_orElse__A1, _orElse__B1]) -> scala.PartialFunction[_orElse__A1, _orElse__B1]: ...
    def par(self) -> scala.collection.parallel.mutable.ParTrieMap[_TrieMap__K, _TrieMap__V]: ...
    def parCombiner(self) -> scala.collection.parallel.Combiner[scala.Tuple2[_TrieMap__K, _TrieMap__V], scala.collection.parallel.mutable.ParTrieMap[_TrieMap__K, _TrieMap__V]]: ...
    def partition(self, p: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], typing.Any]) -> scala.Tuple2['TrieMap'[_TrieMap__K, _TrieMap__V], 'TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    _product__B = typing.TypeVar('_product__B')  # <B>
    def product(self, num: scala.math.Numeric[_product__B]) -> _product__B: ...
    def put(self, key: _TrieMap__K, value: _TrieMap__V) -> scala.Option[_TrieMap__V]: ...
    def putIfAbsent(self, k: _TrieMap__K, v: _TrieMap__V) -> scala.Option[_TrieMap__V]: ...
    def readOnlySnapshot(self) -> scala.collection.Map[_TrieMap__K, _TrieMap__V]: ...
    def readRoot(self, abort: bool) -> 'INode'[_TrieMap__K, _TrieMap__V]: ...
    def readRoot$default$1(self) -> bool: ...
    _reduce__A1 = typing.TypeVar('_reduce__A1')  # <A1>
    def reduce(self, op: scala.Function2[_reduce__A1, _reduce__A1, _reduce__A1]) -> _reduce__A1: ...
    _reduceLeft__B = typing.TypeVar('_reduceLeft__B')  # <B>
    def reduceLeft(self, op: scala.Function2[_reduceLeft__B, scala.Tuple2[_TrieMap__K, _TrieMap__V], _reduceLeft__B]) -> _reduceLeft__B: ...
    _reduceLeftOption__B = typing.TypeVar('_reduceLeftOption__B')  # <B>
    def reduceLeftOption(self, op: scala.Function2[_reduceLeftOption__B, scala.Tuple2[_TrieMap__K, _TrieMap__V], _reduceLeftOption__B]) -> scala.Option[_reduceLeftOption__B]: ...
    _reduceOption__A1 = typing.TypeVar('_reduceOption__A1')  # <A1>
    def reduceOption(self, op: scala.Function2[_reduceOption__A1, _reduceOption__A1, _reduceOption__A1]) -> scala.Option[_reduceOption__A1]: ...
    _reduceRight__B = typing.TypeVar('_reduceRight__B')  # <B>
    def reduceRight(self, op: scala.Function2[scala.Tuple2[_TrieMap__K, _TrieMap__V], _reduceRight__B, _reduceRight__B]) -> _reduceRight__B: ...
    _reduceRightOption__B = typing.TypeVar('_reduceRightOption__B')  # <B>
    def reduceRightOption(self, op: scala.Function2[scala.Tuple2[_TrieMap__K, _TrieMap__V], _reduceRightOption__B, _reduceRightOption__B]) -> scala.Option[_reduceRightOption__B]: ...
    @typing.overload
    def remove(self, k: _TrieMap__K, v: _TrieMap__V) -> bool: ...
    @typing.overload
    def remove(self, k: _TrieMap__K) -> scala.Option[_TrieMap__V]: ...
    @typing.overload
    def replace(self, k: _TrieMap__K, oldvalue: _TrieMap__V, newvalue: _TrieMap__V) -> bool: ...
    @typing.overload
    def replace(self, k: _TrieMap__K, v: _TrieMap__V) -> scala.Option[_TrieMap__V]: ...
    def repr(self) -> typing.Any: ...
    def result(self) -> scala.collection.mutable.Map: ...
    def retain(self, p: scala.Function2[_TrieMap__K, _TrieMap__V, typing.Any]) -> scala.collection.mutable.MapLike[_TrieMap__K, _TrieMap__V, 'TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    def reversed(self) -> scala.collection.immutable.List[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def root(self) -> typing.Any: ...
    def root_$eq(self, x$1: typing.Any) -> None: ...
    _runWith__U = typing.TypeVar('_runWith__U')  # <U>
    def runWith(self, action: scala.Function1[_TrieMap__V, _runWith__U]) -> scala.Function1[_TrieMap__K, typing.Any]: ...
    _sameElements__B = typing.TypeVar('_sameElements__B')  # <B>
    def sameElements(self, that: scala.collection.GenIterable[_sameElements__B]) -> bool: ...
    _scan__B = typing.TypeVar('_scan__B')  # <B>
    _scan__That = typing.TypeVar('_scan__That')  # <That>
    def scan(self, z: _scan__B, op: scala.Function2[_scan__B, _scan__B, _scan__B], cbf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], _scan__B, _scan__That]) -> _scan__That: ...
    _scanLeft__B = typing.TypeVar('_scanLeft__B')  # <B>
    _scanLeft__That = typing.TypeVar('_scanLeft__That')  # <That>
    def scanLeft(self, z: _scanLeft__B, op: scala.Function2[_scanLeft__B, scala.Tuple2[_TrieMap__K, _TrieMap__V], _scanLeft__B], bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], _scanLeft__B, _scanLeft__That]) -> _scanLeft__That: ...
    _scanRight__B = typing.TypeVar('_scanRight__B')  # <B>
    _scanRight__That = typing.TypeVar('_scanRight__That')  # <That>
    def scanRight(self, z: _scanRight__B, op: scala.Function2[scala.Tuple2[_TrieMap__K, _TrieMap__V], _scanRight__B, _scanRight__B], bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], _scanRight__B, _scanRight__That]) -> _scanRight__That: ...
    def seq(self) -> 'TrieMap'[_TrieMap__K, _TrieMap__V]: ...
    def size(self) -> int: ...
    @typing.overload
    def sizeHint(self, size: int) -> None: ...
    @typing.overload
    def sizeHint(self, coll: scala.collection.TraversableLike[typing.Any, typing.Any]) -> None: ...
    @typing.overload
    def sizeHint(self, coll: scala.collection.TraversableLike[typing.Any, typing.Any], delta: int) -> None: ...
    def sizeHintBounded(self, size: int, boundingColl: scala.collection.TraversableLike[typing.Any, typing.Any]) -> None: ...
    def sizeHintIfCheap(self) -> int: ...
    def slice(self, from_: int, until: int) -> typing.Any: ...
    def sliceWithKnownBound(self, from_: int, until: int) -> typing.Any: ...
    def sliceWithKnownDelta(self, from_: int, until: int, delta: int) -> typing.Any: ...
    @typing.overload
    def sliding(self, size: int) -> scala.collection.Iterator['TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    @typing.overload
    def sliding(self, size: int, step: int) -> scala.collection.Iterator['TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    def snapshot(self) -> 'TrieMap'[_TrieMap__K, _TrieMap__V]: ...
    def span(self, p: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], typing.Any]) -> scala.Tuple2['TrieMap'[_TrieMap__K, _TrieMap__V], 'TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    def splitAt(self, n: int) -> scala.Tuple2['TrieMap'[_TrieMap__K, _TrieMap__V], 'TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    def string(self) -> java.lang.String: ...
    def stringPrefix(self) -> java.lang.String: ...
    _sum__B = typing.TypeVar('_sum__B')  # <B>
    def sum(self, num: scala.math.Numeric[_sum__B]) -> _sum__B: ...
    def tail(self) -> typing.Any: ...
    def tails(self) -> scala.collection.Iterator['TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    def take(self, n: int) -> typing.Any: ...
    def takeRight(self, n: int) -> typing.Any: ...
    def takeWhile(self, p: scala.Function1) -> typing.Any: ...
    def thisCollection(self) -> scala.collection.Iterable[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    _to__Col = typing.TypeVar('_to__Col')  # <Col>
    def to(self, cbf: scala.collection.generic.CanBuildFrom[scala.runtime.Nothing., scala.Tuple2[_TrieMap__K, _TrieMap__V], _to__Col]) -> _to__Col: ...
    _toArray__B = typing.TypeVar('_toArray__B')  # <B>
    def toArray(self, evidence$1: scala.reflect.ClassTag[_toArray__B]) -> typing.Any: ...
    _toBuffer__E = typing.TypeVar('_toBuffer__E')  # <E>
    def toBuffer(self) -> scala.collection.mutable.Buffer[_toBuffer__E]: ...
    def toCollection(self, repr: typing.Any) -> scala.collection.Iterable: ...
    def toIndexedSeq(self) -> scala.collection.immutable.IndexedSeq[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def toIterable(self) -> scala.collection.Iterable[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def toIterator(self) -> scala.collection.Iterator[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def toList(self) -> scala.collection.immutable.List[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    _toMap__T = typing.TypeVar('_toMap__T')  # <T>
    _toMap__U = typing.TypeVar('_toMap__U')  # <U>
    def toMap(self, ev: scala.Predef..less.colon.less[scala.Tuple2[_TrieMap__K, _TrieMap__V], scala.Tuple2[_toMap__T, _toMap__U]]) -> scala.collection.immutable.Map[_toMap__T, _toMap__U]: ...
    def toSeq(self) -> scala.collection.Seq[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    _toSet__B = typing.TypeVar('_toSet__B')  # <B>
    def toSet(self) -> scala.collection.immutable.Set[_toSet__B]: ...
    def toStream(self) -> scala.collection.immutable.Stream[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def toString(self) -> java.lang.String: ...
    def toTraversable(self) -> scala.collection.Traversable[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def toVector(self) -> scala.collection.immutable.Vector[scala.Tuple2[_TrieMap__K, _TrieMap__V]]: ...
    def transform(self, f: scala.Function2[_TrieMap__K, _TrieMap__V, _TrieMap__V]) -> scala.collection.mutable.MapLike[_TrieMap__K, _TrieMap__V, 'TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    def transpose(self, asTraversable: scala.Function1) -> scala.collection.GenTraversable: ...
    _unzip__A1 = typing.TypeVar('_unzip__A1')  # <A1>
    _unzip__A2 = typing.TypeVar('_unzip__A2')  # <A2>
    def unzip(self, asPair: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], scala.Tuple2[_unzip__A1, _unzip__A2]]) -> scala.Tuple2[scala.collection.mutable.Iterable[_unzip__A1], scala.collection.mutable.Iterable[_unzip__A2]]: ...
    _unzip3__A1 = typing.TypeVar('_unzip3__A1')  # <A1>
    _unzip3__A2 = typing.TypeVar('_unzip3__A2')  # <A2>
    _unzip3__A3 = typing.TypeVar('_unzip3__A3')  # <A3>
    def unzip3(self, asTriple: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], scala.Tuple3[_unzip3__A1, _unzip3__A2, _unzip3__A3]]) -> scala.Tuple3[scala.collection.mutable.Iterable[_unzip3__A1], scala.collection.mutable.Iterable[_unzip3__A2], scala.collection.mutable.Iterable[_unzip3__A3]]: ...
    def update(self, k: _TrieMap__K, v: _TrieMap__V) -> None: ...
    _updated__V1 = typing.TypeVar('_updated__V1')  # <V1>
    def updated(self, key: _TrieMap__K, value: _updated__V1) -> scala.collection.mutable.Map[_TrieMap__K, _updated__V1]: ...
    def values(self) -> scala.collection.Iterable[_TrieMap__V]: ...
    def valuesIterator(self) -> scala.collection.Iterator[_TrieMap__V]: ...
    @typing.overload
    def view(self) -> scala.collection.IterableView[scala.Tuple2[_TrieMap__K, _TrieMap__V], 'TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    @typing.overload
    def view(self, from_: int, until: int) -> scala.collection.IterableView[scala.Tuple2[_TrieMap__K, _TrieMap__V], 'TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    def withDefault(self, d: scala.Function1[_TrieMap__K, _TrieMap__V]) -> scala.collection.mutable.Map[_TrieMap__K, _TrieMap__V]: ...
    def withDefaultValue(self, d: _TrieMap__V) -> scala.collection.mutable.Map[_TrieMap__K, _TrieMap__V]: ...
    def withFilter(self, p: scala.Function1[scala.Tuple2[_TrieMap__K, _TrieMap__V], typing.Any]) -> scala.collection.generic.FilterMonadic[scala.Tuple2[_TrieMap__K, _TrieMap__V], 'TrieMap'[_TrieMap__K, _TrieMap__V]]: ...
    _zip__A1 = typing.TypeVar('_zip__A1')  # <A1>
    _zip__B = typing.TypeVar('_zip__B')  # <B>
    _zip__That = typing.TypeVar('_zip__That')  # <That>
    def zip(self, that: scala.collection.GenIterable[_zip__B], bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], scala.Tuple2[_zip__A1, _zip__B], _zip__That]) -> _zip__That: ...
    _zipAll__B = typing.TypeVar('_zipAll__B')  # <B>
    _zipAll__A1 = typing.TypeVar('_zipAll__A1')  # <A1>
    _zipAll__That = typing.TypeVar('_zipAll__That')  # <That>
    def zipAll(self, that: scala.collection.GenIterable[_zipAll__B], thisElem: _zipAll__A1, thatElem: _zipAll__B, bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], scala.Tuple2[_zipAll__A1, _zipAll__B], _zipAll__That]) -> _zipAll__That: ...
    _zipWithIndex__A1 = typing.TypeVar('_zipWithIndex__A1')  # <A1>
    _zipWithIndex__That = typing.TypeVar('_zipWithIndex__That')  # <That>
    def zipWithIndex(self, bf: scala.collection.generic.CanBuildFrom['TrieMap'[_TrieMap__K, _TrieMap__V], scala.Tuple2[_zipWithIndex__A1, typing.Any], _zipWithIndex__That]) -> _zipWithIndex__That: ...
    class MangledHashing(scala.util.hashing.Hashing[_TrieMap__MangledHashing__K], typing.Generic[_TrieMap__MangledHashing__K]):
        def __init__(self): ...
        def hash(self, k: _TrieMap__MangledHashing__K) -> int: ...

class INodeBase: ...

_INode__K = typing.TypeVar('_INode__K')  # <K>
_INode__V = typing.TypeVar('_INode__V')  # <V>
class INode(INodeBase[_INode__K, _INode__V], typing.Generic[_INode__K, _INode__V]):
    @typing.overload
    def __init__(self, g: 'Gen'): ...
    @typing.overload
    def __init__(self, bn: MainNode[_INode__K, _INode__V], g: 'Gen'): ...
    def CAS(self, old: MainNode[_INode__K, _INode__V], n: MainNode[_INode__K, _INode__V]) -> bool: ...
    def GCAS(self, old: MainNode[_INode__K, _INode__V], n: MainNode[_INode__K, _INode__V], ct: TrieMap[_INode__K, _INode__V]) -> bool: ...
    def GCAS_READ(self, ct: TrieMap[_INode__K, _INode__V]) -> MainNode[_INode__K, _INode__V]: ...
    @staticmethod
    def KEY_ABSENT() -> typing.Any: ...
    @staticmethod
    def KEY_PRESENT() -> typing.Any: ...
    def WRITE(self, nval: MainNode[_INode__K, _INode__V]) -> None: ...
    def cachedSize(self, ct: TrieMap[_INode__K, _INode__V]) -> int: ...
    def copyToGen(self, ngen: 'Gen', ct: TrieMap[_INode__K, _INode__V]) -> 'INode'[_INode__K, _INode__V]: ...
    def gcasRead(self, ct: TrieMap[_INode__K, _INode__V]) -> MainNode[_INode__K, _INode__V]: ...
    def isNullInode(self, ct: TrieMap[_INode__K, _INode__V]) -> bool: ...
    _newRootNode__K = typing.TypeVar('_newRootNode__K')  # <K>
    _newRootNode__V = typing.TypeVar('_newRootNode__V')  # <V>
    @staticmethod
    def newRootNode() -> 'INode'[_newRootNode__K, _newRootNode__V]: ...
    def rec_insert(self, k: _INode__K, v: _INode__V, hc: int, lev: int, parent: 'INode'[_INode__K, _INode__V], startgen: 'Gen', ct: TrieMap[_INode__K, _INode__V]) -> bool: ...
    def rec_insertif(self, k: _INode__K, v: _INode__V, hc: int, cond: typing.Any, lev: int, parent: 'INode'[_INode__K, _INode__V], startgen: 'Gen', ct: TrieMap[_INode__K, _INode__V]) -> scala.Option[_INode__V]: ...
    def rec_lookup(self, k: _INode__K, hc: int, lev: int, parent: 'INode'[_INode__K, _INode__V], startgen: 'Gen', ct: TrieMap[_INode__K, _INode__V]) -> typing.Any: ...
    def rec_remove(self, k: _INode__K, v: _INode__V, hc: int, lev: int, parent: 'INode'[_INode__K, _INode__V], startgen: 'Gen', ct: TrieMap[_INode__K, _INode__V]) -> scala.Option[_INode__V]: ...
    def string(self, lev: int) -> java.lang.String: ...

class Gen: ...

_CNode__K = typing.TypeVar('_CNode__K')  # <K>
_CNode__V = typing.TypeVar('_CNode__V')  # <V>
class CNode(scala.collection.concurrent.CNodeBase[_CNode__K, _CNode__V], typing.Generic[_CNode__K, _CNode__V]):
    def __init__(self, bitmap: int, array: typing.List[BasicNode], gen: Gen): ...
    def array(self) -> typing.List[BasicNode]: ...
    def bitmap(self) -> int: ...
    def cachedSize(self, ct: typing.Any) -> int: ...
    _dual__K = typing.TypeVar('_dual__K')  # <K>
    _dual__V = typing.TypeVar('_dual__V')  # <V>
    @staticmethod
    def dual(x: SNode[_dual__K, _dual__V], xhc: int, y: SNode[_dual__K, _dual__V], yhc: int, lev: int, gen: Gen) -> MainNode[_dual__K, _dual__V]: ...
    def gen(self) -> Gen: ...
    def insertedAt(self, pos: int, flag: int, nn: BasicNode, gen: Gen) -> 'CNode'[_CNode__K, _CNode__V]: ...
    def removedAt(self, pos: int, flag: int, gen: Gen) -> 'CNode'[_CNode__K, _CNode__V]: ...
    def renewed(self, ngen: Gen, ct: TrieMap[_CNode__K, _CNode__V]) -> 'CNode'[_CNode__K, _CNode__V]: ...
    def string(self, lev: int) -> java.lang.String: ...
    def toCompressed(self, ct: TrieMap[_CNode__K, _CNode__V], lev: int, gen: Gen) -> MainNode[_CNode__K, _CNode__V]: ...
    def toContracted(self, lev: int) -> MainNode[_CNode__K, _CNode__V]: ...
    def toString(self) -> java.lang.String: ...
    def updatedAt(self, pos: int, nn: BasicNode, gen: Gen) -> 'CNode'[_CNode__K, _CNode__V]: ...

class CNodeBase: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("scala.collection.concurrent")``.

    BasicNode: typing.Type[BasicNode]
    CNode: typing.Type[CNode]
    CNodeBase: typing.Type[CNodeBase]
    Debug: typing.Type[Debug]
    FailedNode: typing.Type[FailedNode]
    Gen: typing.Type[Gen]
    INode: typing.Type[INode]
    INodeBase: typing.Type[INodeBase]
    KVNode: typing.Type[KVNode]
    LNode: typing.Type[LNode]
    MainNode: typing.Type[MainNode]
    Map: typing.Type[Map]
    RDCSS_Descriptor: typing.Type[RDCSS_Descriptor]
    RestartException: typing.Type[RestartException]
    SNode: typing.Type[SNode]
    TNode: typing.Type[TNode]
    TrieMap: typing.Type[TrieMap]
    TrieMapIterator: typing.Type[TrieMapIterator]
    TrieMapSerializationEnd: typing.Type[TrieMapSerializationEnd]



import java.lang
import org.bouncycastle.crypto
import org.bouncycastle.crypto.digests
import scala.collection
import scala.math
import scala.util
import scorex
import supertagged
import typing



class Blake2b256:
    @staticmethod
    def DigestSize() -> int: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @staticmethod
    def byteArrayToDigest(bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def prefixedHash(prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

class Blake2b512:
    @staticmethod
    def DigestSize() -> int: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @staticmethod
    def byteArrayToDigest(bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def prefixedHash(prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

_CryptographicHash__D = typing.TypeVar('_CryptographicHash__D', bound=typing.List[int])  # <D>
class CryptographicHash(typing.Generic[_CryptographicHash__D]):
    @staticmethod
    def $init$($this: 'CryptographicHash') -> None: ...
    def DigestSize(self) -> int: ...
    @typing.overload
    def apply(self, input: typing.List[int]) -> _CryptographicHash__D: ...
    @typing.overload
    def apply(self, input: typing.Union[java.lang.String, str]) -> _CryptographicHash__D: ...
    def byteArrayToDigest(self, bytes: typing.List[int]) -> scala.util.Try[_CryptographicHash__D]: ...
    @typing.overload
    def hash(self, input: typing.List[int]) -> _CryptographicHash__D: ...
    @typing.overload
    def hash(self, input: typing.Union[java.lang.String, str]) -> _CryptographicHash__D: ...
    @typing.overload
    def hash(self, inputs: scala.collection.Seq[typing.List[int]]) -> _CryptographicHash__D: ...
    def prefixedHash(self, prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> _CryptographicHash__D: ...

class Keccak256:
    @staticmethod
    def DigestSize() -> int: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @staticmethod
    def byteArrayToDigest(bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def prefixedHash(prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

class Keccak512:
    @staticmethod
    def DigestSize() -> int: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @staticmethod
    def byteArrayToDigest(bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def prefixedHash(prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

class Sha256:
    @staticmethod
    def DigestSize() -> int: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @staticmethod
    def byteArrayToDigest(bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def prefixedHash(prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

class Skein256:
    @staticmethod
    def DigestSize() -> int: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @staticmethod
    def byteArrayToDigest(bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def prefixedHash(prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

class Skein512:
    @staticmethod
    def DigestSize() -> int: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @staticmethod
    def byteArrayToDigest(bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def prefixedHash(prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

class Stribog256:
    @staticmethod
    def DigestSize() -> int: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @staticmethod
    def byteArrayToDigest(bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def prefixedHash(prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

class Stribog512:
    @staticmethod
    def DigestSize() -> int: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @staticmethod
    def byteArrayToDigest(bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def prefixedHash(prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

class Whirlpool:
    @staticmethod
    def DigestSize() -> int: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def apply(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @staticmethod
    def byteArrayToDigest(bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    @staticmethod
    def hash(inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    @staticmethod
    def prefixedHash(prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

_BouncyCastleHash__D = typing.TypeVar('_BouncyCastleHash__D', bound=typing.List[int])  # <D>
class BouncyCastleHash(CryptographicHash[_BouncyCastleHash__D], typing.Generic[_BouncyCastleHash__D]):
    @staticmethod
    def $init$($this: 'BouncyCastleHash') -> None: ...
    def digestFn(self) -> org.bouncycastle.crypto.ExtendedDigest: ...
    def internalHash(self, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    def internalPrefixedHash(self, prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...

_CommutativeHash__D = typing.TypeVar('_CommutativeHash__D', bound=typing.List[int])  # <D>
class CommutativeHash(CryptographicHash[_CommutativeHash__D], typing.Generic[_CommutativeHash__D]):
    def __init__(self, hf: CryptographicHash[_CommutativeHash__D]): ...
    def DigestSize(self) -> int: ...
    @typing.overload
    def apply(self, input: typing.List[int]) -> _CommutativeHash__D: ...
    @typing.overload
    def apply(self, x: typing.List[int], y: typing.List[int]) -> _CommutativeHash__D: ...
    @typing.overload
    def apply(self, input: typing.Union[java.lang.String, str]) -> _CommutativeHash__D: ...
    def byteArrayToDigest(self, bytes: typing.List[int]) -> scala.util.Try[_CommutativeHash__D]: ...
    @typing.overload
    def hash(self, input: typing.List[int]) -> _CommutativeHash__D: ...
    @typing.overload
    def hash(self, x: typing.List[int], y: typing.List[int]) -> _CommutativeHash__D: ...
    @typing.overload
    def hash(self, input: typing.Union[java.lang.String, str]) -> _CommutativeHash__D: ...
    @typing.overload
    def hash(self, inputs: scala.collection.Seq[typing.List[int]]) -> _CommutativeHash__D: ...
    @typing.overload
    def prefixedHash(self, prefix: int, x: typing.List[int], y: typing.List[int]) -> _CommutativeHash__D: ...
    @typing.overload
    def prefixedHash(self, prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> _CommutativeHash__D: ...

class CryptographicHash32(CryptographicHash[typing.List[int]]):
    @staticmethod
    def $init$($this: 'CryptographicHash32') -> None: ...
    def DigestSize(self) -> int: ...
    def byteArrayToDigest(self, bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    def scorex$crypto$hash$CryptographicHash32$_setter_$DigestSize_$eq(self, x$1: int) -> None: ...

class CryptographicHash64(CryptographicHash[typing.List[int]]):
    @staticmethod
    def $init$($this: 'CryptographicHash64') -> None: ...
    def DigestSize(self) -> int: ...
    def byteArrayToDigest(self, bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    def scorex$crypto$hash$CryptographicHash64$_setter_$DigestSize_$eq(self, x$1: int) -> None: ...

_ThreadUnsafeHash__D = typing.TypeVar('_ThreadUnsafeHash__D', bound=typing.List[int])  # <D>
class ThreadUnsafeHash(CryptographicHash[_ThreadUnsafeHash__D], typing.Generic[_ThreadUnsafeHash__D]): ...

_Blake2b__D = typing.TypeVar('_Blake2b__D', bound=typing.List[int])  # <D>
class Blake2b(BouncyCastleHash[_Blake2b__D], typing.Generic[_Blake2b__D]):
    @staticmethod
    def $init$($this: 'Blake2b') -> None: ...
    @typing.overload
    def digestFn(self) -> org.bouncycastle.crypto.ExtendedDigest: ...
    @typing.overload
    def digestFn(self) -> org.bouncycastle.crypto.digests.Blake2bDigest: ...

class Blake2b256Unsafe(CryptographicHash32, ThreadUnsafeHash[typing.List[int]]):
    def __init__(self): ...
    def DigestSize(self) -> int: ...
    @typing.overload
    def apply(self, input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    def apply(self, input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    def byteArrayToDigest(self, bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    def hash(self, input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    def hash(self, input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    def hash(self, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    def prefixedHash(self, prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    def scorex$crypto$hash$CryptographicHash32$_setter_$DigestSize_$eq(self, x$1: int) -> None: ...

_Keccak__D = typing.TypeVar('_Keccak__D', bound=typing.List[int])  # <D>
class Keccak(BouncyCastleHash[_Keccak__D], typing.Generic[_Keccak__D]):
    @staticmethod
    def $init$($this: 'Keccak') -> None: ...
    @typing.overload
    def digestFn(self) -> org.bouncycastle.crypto.ExtendedDigest: ...
    @typing.overload
    def digestFn(self) -> org.bouncycastle.crypto.digests.KeccakDigest: ...

class Sha256Unsafe(CryptographicHash32, ThreadUnsafeHash[typing.List[int]]):
    def __init__(self): ...
    def DigestSize(self) -> int: ...
    @typing.overload
    def apply(self, input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    def apply(self, input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    def byteArrayToDigest(self, bytes: typing.List[int]) -> scala.util.Try[typing.List[int]]: ...
    @typing.overload
    def hash(self, input: typing.List[int]) -> typing.List[int]: ...
    @typing.overload
    def hash(self, input: typing.Union[java.lang.String, str]) -> typing.List[int]: ...
    @typing.overload
    def hash(self, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    def prefixedHash(self, prefix: int, inputs: scala.collection.Seq[typing.List[int]]) -> typing.List[int]: ...
    def scorex$crypto$hash$CryptographicHash32$_setter_$DigestSize_$eq(self, x$1: int) -> None: ...

class package:
    class BaseDigest(supertagged.package.TaggedType[typing.List[int]]): ...
    class Digest32$(scorex.crypto.hash.package.BaseDigest):
        MODULE$: typing.ClassVar['package.Digest32.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class Digest64$(scorex.crypto.hash.package.BaseDigest):
        MODULE$: typing.ClassVar['package.Digest64.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...
    class NonStandardDigest$(scorex.crypto.hash.package.BaseDigest):
        MODULE$: typing.ClassVar['package.NonStandardDigest.'] = ...
        def __init__(self): ...
        _$at$at__TagIn = typing.TypeVar('_$at$at__TagIn')  # <TagIn>
        _$at$at__Sub = typing.TypeVar('_$at$at__Sub')  # <Sub>
        _$at$at__C = typing.TypeVar('_$at$at__C')  # <C>
        def $at$at(self, c: _.at.at__C, tagger: supertagged.package.Tagger[_.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.at.at__Sub, _.at.at__C]) -> typing.Any: ...
        _$bang$at$at__TagIn = typing.TypeVar('_$bang$at$at__TagIn')  # <TagIn>
        _$bang$at$at__Sub = typing.TypeVar('_$bang$at$at__Sub')  # <Sub>
        _$bang$at$at__C = typing.TypeVar('_$bang$at$at__C')  # <C>
        def $bang$at$at(self, c: _.bang.at.at__C, tagger: supertagged.package.Tagger[_.bang.at.at__TagIn, typing.List[int], supertagged.package.TaggedType, _.bang.at.at__Sub, _.bang.at.at__C]) -> typing.Any: ...
        _apply__TagIn = typing.TypeVar('_apply__TagIn')  # <TagIn>
        _apply__Sub = typing.TypeVar('_apply__Sub')  # <Sub>
        _apply__C = typing.TypeVar('_apply__C')  # <C>
        def apply(self, c: _apply__C, tagger: supertagged.package.Tagger[_apply__TagIn, typing.List[int], supertagged.package.TaggedType, _apply__Sub, _apply__C]) -> typing.Any: ...
        _ordering__U = typing.TypeVar('_ordering__U')  # <U>
        def ordering(self, origin: scala.math.Ordering[typing.List[int]]) -> scala.math.Ordering[typing.List[int]]: ...
        def raw(self, c: typing.Any) -> typing.Any: ...
        _untag__TagIn = typing.TypeVar('_untag__TagIn')  # <TagIn>
        _untag__Sub = typing.TypeVar('_untag__Sub')  # <Sub>
        _untag__C = typing.TypeVar('_untag__C')  # <C>
        def untag(self, c: _untag__C, tagger: supertagged.package.Tagger[_untag__TagIn, typing.List[int], supertagged.package.TaggedType, _untag__Sub, _untag__C]) -> typing.Any: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("scorex.crypto.hash")``.

    Blake2b: typing.Type[Blake2b]
    Blake2b256: typing.Type[Blake2b256]
    Blake2b256Unsafe: typing.Type[Blake2b256Unsafe]
    Blake2b512: typing.Type[Blake2b512]
    BouncyCastleHash: typing.Type[BouncyCastleHash]
    CommutativeHash: typing.Type[CommutativeHash]
    CryptographicHash: typing.Type[CryptographicHash]
    CryptographicHash32: typing.Type[CryptographicHash32]
    CryptographicHash64: typing.Type[CryptographicHash64]
    Keccak: typing.Type[Keccak]
    Keccak256: typing.Type[Keccak256]
    Keccak512: typing.Type[Keccak512]
    Sha256: typing.Type[Sha256]
    Sha256Unsafe: typing.Type[Sha256Unsafe]
    Skein256: typing.Type[Skein256]
    Skein512: typing.Type[Skein512]
    Stribog256: typing.Type[Stribog256]
    Stribog512: typing.Type[Stribog512]
    ThreadUnsafeHash: typing.Type[ThreadUnsafeHash]
    Whirlpool: typing.Type[Whirlpool]
    package: typing.Type[package]



import json
from os import times
import typing

import jpype
import jpype.imports
from jpype.types import *
from jpype import JImplements, JOverride
from enum import Enum
from typing import Dict, List, TypeVar

try:
    jpype.addClassPath('../jars/*')
    jpype.addClassPath('./jars/*')
    jpype.startJVM(None, convertStrings=True)
except OSError:
    print("JVM already running")


from org.ergoplatform import ErgoAddress, ErgoAddressEncoder
from org.ergoplatform.appkit import Address, BoxOperations, Eip4Token, ErgoClientException, ErgoContract, ErgoToken, ErgoType, ErgoValue, ExplorerAndPoolUnspentBoxesLoader, InputBox, Iso, JavaHelpers, NetworkType, OutBox, PreHeader, ReducedTransaction, RestApiErgoClient, SignedTransaction, UnsignedTransaction, SigmaProp, ErgoAuthUtils
from org.ergoplatform.restapi.client import ApiClient, ErgoTransactionOutput, ErgoTransactionUnsignedInput, TransactionSigningRequest, UnsignedErgoTransaction, UtxoApi, WalletApi
from org.ergoplatform.explorer.client import ExplorerApiClient
from org.ergoplatform.appkit.impl import BlockchainContextBuilderImpl, BlockchainContextImpl, ErgoTreeContract, InputBoxImpl, ScalaBridge, SignedTransactionImpl, UnsignedTransactionImpl
from sigmastate.Values import ErgoTree
from java.lang.reflect import UndeclaredThrowableException
from sigmastate.serialization import ErgoTreeSerializer
from sigmastate.lang.exceptions import InterpreterException
from retrofit2 import Response
from java.util import ArrayList
import scala
from java.lang import NullPointerException
from java.util.function import Function
import base64


#region LOGGING
import logging

import inspect
myself = lambda: inspect.stack()[1][3]
#endregion LOGGING


class ErgoException(Exception):
    pass

class ErgoValueT(Enum):
    Long = 0,
    ByteArray = 1,
    LongArray = 2,
    ByteArrayFromHex = 4

class ErgoAppKit:
    
    def __init__(self, nodeUrl: str, networkType: str, explorerUrl: str, nodeApiKey: str = ""):
        self._nodeUrl = nodeUrl
        self._networkType = ErgoAppKit.NetworkType(networkType)
        self._client: ApiClient = ApiClient(self._nodeUrl, "ApiKeyAuth", nodeApiKey)
        self._explorerUrl = explorerUrl.replace("/api/v1","")
        self._ergoClient = RestApiErgoClient.create(nodeUrl,self._networkType,nodeApiKey,self._explorerUrl)
        if self._explorerUrl!="":
            self._explorer = ExplorerApiClient(self._explorerUrl)
        else:
            self._explorer = None
        self._addrEnc = ErgoAddressEncoder(self._networkType.networkPrefix)

    def compileErgoScript(self, ergoScript: str, constants: Dict[str,typing.Any] = {}) -> ErgoTree:
        return JavaHelpers.compile(constants,ergoScript,self._networkType.networkPrefix)

    def tree2Address(self, ergoTree):
        return self._addrEnc.fromProposition(ergoTree).get().toString()

    @JImplements(Function)
    class BuildOutBoxExecutor(object):

        def __init__(self, value: int, tokens: Dict[str,int], registers, contract):
            self._value = value
            self._tokens = tokens
            self._registers = registers
            self._contract = contract

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> OutBox:
            tb = ctx.newTxBuilder()
            ergoTokens = []
            tb = tb.outBoxBuilder().contract(self._contract).value(self._value)
            if self._tokens is not None:
                for token in self._tokens.keys():
                    ergoTokens.append(ErgoToken(token,self._tokens[token]))
                tb = tb.tokens(ergoTokens)

            if self._registers is not None:
                tb = tb.registers(self._registers)

            return tb.build()

    def buildOutBox(self,value: int, tokens: Dict[str,int], registers, contract) -> OutBox:
        return self._ergoClient.execute(ErgoAppKit.BuildOutBoxExecutor(value,tokens,registers,contract))

    @JImplements(Function)
    class GetBoxesByIdExecutor(object):

        def __init__(self, boxIds: List[str], api: UtxoApi):
            self._boxIds = boxIds
            self._api = api

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> List[InputBox]:
            boxData: ErgoTransactionOutput = None
            res = []
            for id in self._boxIds:
                response: Response = self._api.getBoxWithPoolById(id).execute()
                if response.isSuccessful():
                    boxData = response.body()
                if boxData is None:
                    raise ErgoClientException("Cannot load UTXO box " + id, None)
                res.append(InputBoxImpl(boxData))
            return ArrayList(res)

    def getBoxesById(self, boxIds: List[str]) -> List[InputBox]:
        api = self._client.createService(UtxoApi)
        return list(self._ergoClient.execute(ErgoAppKit.GetBoxesByIdExecutor(boxIds,api)))
    
    @JImplements(Function)
    class MintTokenExecutor(object):

        def __init__(self, value: int, tokenId: str, tokenName: str, tokenDesc: str, mintAmount: int, decimals: int, contract: ErgoContract):
            self._value = value
            self._tokenId = tokenId
            self._tokenName = tokenName
            self._tokenDesc = tokenDesc
            self._mintAmount = mintAmount
            self._decimals = decimals
            self._contract = contract

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> OutBox:
            tb = ctx.newTxBuilder()
            return tb.outBoxBuilder().contract(self._contract).value(self._value).mintToken(Eip4Token(self._tokenId,self._mintAmount,self._tokenName,self._tokenDesc,self._decimals)).build()

    def mintToken(self, value: int, tokenId: str, tokenName: str, tokenDesc: str, mintAmount: int, decimals: int, contract: ErgoContract) -> OutBox:
        return self._ergoClient.execute(ErgoAppKit.MintTokenExecutor(value,tokenId,tokenName,tokenDesc,mintAmount,decimals,contract))

    def buildInputBox(self,value: int, tokens: Dict[str,int], registers, contract, withTxId: str = "ce552663312afc2379a91f803c93e2b10b424f176fbc930055c10def2fd88a5d") -> InputBox:
        return self.buildOutBox(value, tokens, registers, contract).convertToInputWith(withTxId, 0)

    @JImplements(Function)
    class BoxesToSpendFromListExecutor(object):

        def __init__(self,   addresses: list[str], nergToSpend: int, tokensToSpend: Dict[str,int] = {}, includeMempool: bool = True):
            self._addresses = addresses
            self._nergToSpend = nergToSpend
            self._tokensToSpend = tokensToSpend
            self._includeMempool = includeMempool

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> List[InputBox]:
            tts = ArrayList(ErgoAppKit.mapToErgoTokenList(self._tokensToSpend))
            nergLeft = self._nergToSpend
            addresses = []
            for addr in self._addresses:
                addresses.append(Address.create(addr))
            result = BoxOperations.createForSenders(ArrayList(addresses),ctx).withTokensToSpend(tts).withAmountToSpend(nergLeft).withInputBoxesLoader(ExplorerAndPoolUnspentBoxesLoader().withAllowChainedTx(self._includeMempool)).loadTop()
            if ErgoAppKit.boxesCovered(result,self._nergToSpend,self._tokensToSpend):
                return ArrayList(result)
            else:
                return None

    def boxesToSpendFromList(self, addresses: list[str], nergToSpend: int, tokensToSpend: Dict[str,int] = {}, includeMempool: bool = True) -> List[InputBox]:
        return list(self._ergoClient.execute(ErgoAppKit.BoxesToSpendFromListExecutor(addresses,nergToSpend,tokensToSpend,includeMempool)))

    def boxesToSpend(self, address: str, nergToSpend: int, tokensToSpend: Dict[str,int] = {}, includeMempool: bool = True) -> List[InputBox]:
        return list(self._ergoClient.execute(ErgoAppKit.BoxesToSpendFromListExecutor([address],nergToSpend,tokensToSpend,includeMempool)))

    def ergoValue(value, t: ErgoValueT):
        if t == ErgoValueT.Long:
            res = ErgoValue.of(JLong(value))
            return res
        if t == ErgoValueT.ByteArrayFromHex:
            return ErgoValue.of(bytes.fromhex(value), ErgoType.byteType())
        if t == ErgoValueT.LongArray:
            collVal = []
            for l in value:
                collVal.append(JLong(l))
            res = ErgoValue.of(collVal,ErgoType.longType())
            return res
        if t == ErgoValueT.ByteArray:
            res = ErgoValue.of(value, ErgoType.byteType())
            return res

    def dummyContract(self) -> ErgoContract:
        return ErgoTreeContract(Address.create("4MQyML64GnzMxZgm").getErgoAddress().script(),self._networkType)

    def contractFromTree(self,tree: ErgoTree) -> ErgoContract:
        return ErgoTreeContract(tree,self._networkType)

    def contractFromAddress(self,addr: str) -> ErgoContract:
        return ErgoTreeContract(Address.create(addr).getErgoAddress().script(),self._networkType)

    def treeFromBytes(self,bytes: bytes) -> ErgoTree:
        treeSerializer = ErgoTreeSerializer()
        return self._addrEnc.fromProposition(treeSerializer.deserializeErgoTree(bytes)).get().script()

    @JImplements(Function)
    class PreHeaderExecutor(object):

        def __init__(self,  timestamp: int = None):
            self._timestamp = timestamp

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> PreHeader:
            phb = ctx.createPreHeader()
            if self._timestamp is not None:
                phb = phb.timestamp(JLong(self._timestamp))
            return phb.build()

    def preHeader(self, timestamp: int = None) -> PreHeader:
        return self._ergoClient.execute(ErgoAppKit.PreHeaderExecutor(timestamp))

    @JImplements(Function)
    class BuildUnsignedTransactionExecutor(object):

        def __init__(self,  inputs: List[InputBox], outputs: List[OutBox], fee: int, sendChangeTo: ErgoAddress, tokensToBurn: Dict[str,int] = None, dataInputs: List[InputBox] = None, preHeader: PreHeader = None):
            self._inputs = inputs
            self._outputs = outputs
            self._fee = fee
            self._sendChangeTo = sendChangeTo
            self._tokensToBurn = tokensToBurn
            self._dataInputs = dataInputs
            self._preHeader = preHeader

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> UnsignedTransaction:
            tb = ctx.newTxBuilder()
            if self._preHeader is not None:
                tb = tb.preHeader(self._preHeader)
            if self._dataInputs is not None:
                tb = tb.withDataInputs(ArrayList(self._dataInputs))
            if self._tokensToBurn is not None:
                tb = tb.tokensToBurn(ErgoAppKit.mapToErgoTokenList(self._tokensToBurn))
            tb = tb.boxesToSpend(ArrayList(self._inputs)).fee(self._fee).outputs(self._outputs).sendChangeTo(self._sendChangeTo)
            return tb.build()

    def buildUnsignedTransaction(self, inputs: List[InputBox], outputs: List[OutBox], fee: int, sendChangeTo: ErgoAddress, tokensToBurn: Dict[str,int] = None, dataInputs: List[InputBox] = None, preHeader: PreHeader = None) -> UnsignedTransaction:
        return self._ergoClient.execute(ErgoAppKit.BuildUnsignedTransactionExecutor(inputs,outputs,fee,sendChangeTo,tokensToBurn,dataInputs,preHeader))

    @JImplements(Function)
    class SignTransactionExecutor(object):

        def __init__(self, unsignedTx: UnsignedTransaction):
            self._unsignedTx = unsignedTx

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> SignedTransaction:
            prover = ctx.newProverBuilder().build()
            return prover.sign(self._unsignedTx)

    def signTransaction(self, unsignedTx: UnsignedTransaction) -> SignedTransaction:
        try:
            return self._ergoClient.execute(ErgoAppKit.SignTransactionExecutor(unsignedTx))
        except UndeclaredThrowableException as e:
            raise e.getCause()

    @JImplements(Function)
    class SendTransactionExecutor(object):

        def __init__(self, signedTx: SignedTransaction):
            self._signedTx = signedTx

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> str:
            return ctx.sendTransaction(self._signedTx)

    def sendTransaction(self, signedTx: SignedTransaction) -> str:
        return self._ergoClient.execute(ErgoAppKit.SendTransactionExecutor(signedTx))

    @JImplements(Function)
    class SignTransactionWithNodeExecutor(object):

        def __init__(self, unsignedTx: UnsignedTransactionImpl, api: WalletApi):
            self._unsignedTx = unsignedTx
            self._api = api

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> SignedTransaction:
            signRequest = TransactionSigningRequest()
            unsignedErgoTx = UnsignedErgoTransaction()
            unsignedErgoLikeTx = self._unsignedTx.getTx()
            for i in range(int(unsignedErgoLikeTx.inputs().length())):
                input = unsignedErgoLikeTx.inputs().apply(JInt(i))
                unsignedInput = ErgoTransactionUnsignedInput()
                unsignedInput.setBoxId(self._unsignedTx.getInputs()[i].getId().toString())
                unsignedInput.setExtension(scala.collection.JavaConversions.mapAsJavaMap(input.extension().values()))
                unsignedErgoTx = unsignedErgoTx.addInputsItem(unsignedInput)
            if unsignedErgoLikeTx.dataInputs().length() > 0:
                unsignedErgoTx.setDataInputs(Iso.inverseIso(Iso.JListToIndexedSeq(ScalaBridge.isoErgoTransactionDataInput())).to(unsignedErgoLikeTx.dataInputs()))
            unsignedErgoTx.setOutputs(Iso.inverseIso(Iso.JListToIndexedSeq(ScalaBridge.isoErgoTransactionOutput())).to(unsignedErgoLikeTx.outputs()))
            signRequest.setTx(unsignedErgoTx)
            response = self._api.walletTransactionSign(signRequest).execute()
            if response.isSuccessful():
                ergoTx = response.body()
                tx = ScalaBridge.isoErgoTransaction().to(ergoTx)
                signedTx = SignedTransactionImpl(ctx,tx,0)
                return signedTx
            else:
                error = json.loads(response.errorBody().string())
                raise ErgoException(f'{error["reason"]}: {error["detail"]}')

    def signTransactionWithNode(self, unsignedTx: UnsignedTransactionImpl) -> SignedTransaction:
        api = self._client.createService(WalletApi)
        return self._ergoClient.execute(ErgoAppKit.SignTransactionWithNodeExecutor(unsignedTx,api))

    @JImplements(Function)
    class GetUnspentBoxesExecutor(object):

        def __init__(self, address: str):
            self._address = address

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> List[InputBox]:
            addr = Address.create(self._address)
            offset = 0
            limit = 100
            result = []
            pageResult = ctx.getUnspentBoxesFor(addr,offset,limit)
            while len(pageResult) > 0:
                result = result + list(pageResult)
                offset += limit
                pageResult = ctx.getUnspentBoxesFor(addr,offset,limit)
            return ArrayList(result)

    def getUnspentBoxes(self, address: str) -> List[InputBox]:
        return list(self._ergoClient.execute(ErgoAppKit.GetUnspentBoxesExecutor(address)))

    @JImplements(Function)
    class ReducedTxExecutor(object):

        def __init__(self, unsignedTx: UnsignedTransactionImpl):
            self._unsignedTx = unsignedTx

        @JOverride
        def apply(self, ctx: BlockchainContextImpl) -> ReducedTransaction:
            return ctx.newProverBuilder().build().reduce(self._unsignedTx,0)

    def reducedTx(self, unsignedTx: UnsignedTransactionImpl) -> ReducedTransaction:
        return self._ergoClient.execute(ErgoAppKit.ReducedTxExecutor(unsignedTx))

    #static functions
    def NetworkType(networkType: str):
        if networkType.lower()=="testnet":
            return NetworkType.TESTNET
        else:
            return NetworkType.MAINNET

    def formErgoPaySigningRequest(
            reducedTx: ReducedTransaction, 
            address: str = None, 
            message: str = None,
            messageSeverity: str = None,
            replyTo: str = None
        ) -> str:

        result = {}
        result['reducedTx'] = base64.urlsafe_b64encode(reducedTx.toBytes()).decode()
        if address is not None: result['address'] = address
        if message is not None: result['message'] = message
        if messageSeverity is not None: result['messageSeverity'] = messageSeverity
        if replyTo is not None: result['replyTo'] = replyTo
        return result

    def unsignedTxToJson(unsignedTx: UnsignedTransactionImpl) -> str:
        inputs = []
        for i in unsignedTx.getInputs():
            j = json.loads(i.toJson(False))
            j["extension"] = {}
            j["value"] = str(j["value"])
            for ass in j["assets"]:
                ass["amount"] = str(ass["amount"])
            inputs.append(j)
        dataInputs = []
        for di in unsignedTx.getDataInputs():
            j = json.loads(di.toJson(False))
            j["extension"] = {}
            j["value"] = str(j["value"])
            for ass in j["assets"]:
                ass["amount"] = str(ass["amount"])
            dataInputs.append(j)
        outputs = []
        for o in unsignedTx.getOutputs():
            assets = []
            for t in o.getTokens():
                assets.append({'tokenId': str(t.getId().toString()), 'amount': str(t.getValue())})  
            additionalRegisters = {}
            r = 4
            for additionalRegister in o.getRegisters():
                additionalRegisters[f'R{r}']=additionalRegister.toHex()
                r+=1
            outputs.append({
                'value': str(o.getValue()),
                'ergoTree': o.getErgoTree().bytesHex(),
                'assets': assets,
                'additionalRegisters': additionalRegisters,
                'creationHeight': o.getCreationHeight()
            })

        return {
            'inputs': inputs,
            'dataInputs': dataInputs,
            'outputs': outputs
        }

    def cutOffExcessUTXOs(utxos: List[InputBox], nErgRequired: int, tokensToSpend: Dict[str,int]) -> List[InputBox]:
        result = []
        for utxo in utxos:
            result.append(utxo)
            if ErgoAppKit.boxesCovered(result,nErgRequired,tokensToSpend):
                return result
        return result

    def boxesCovered(inputs: List[InputBox], nErgRequired: int, tokensToSpend: Dict[str,int]) -> bool:
        balance = ErgoAppKit.getBalance(inputs)
        changeRequired = False
        if balance["erg"] < nErgRequired:
            return False
        if balance["erg"] - nErgRequired > 0 and balance["erg"] - nErgRequired < int(1e6):
            return False
        for token in list(tokensToSpend.keys()):
            if balance.get(token,0) < tokensToSpend[token]:
                return False
            if balance.get(token,0) > tokensToSpend[token]:
                changeRequired = True
        if len(tokensToSpend.keys()) < len(balance.keys()) - 1:
            changeRequired = True
        if changeRequired:
            return balance["erg"] > nErgRequired + int(1e6)
        return True

    def deserializeLongArray(hex: str) -> List[int]:
        ergoValue = ErgoValue.fromHex(hex)
        res = []
        for val in ergoValue.getValue().toArray():
            res.append(val)
        return res

    def getBalance(boxes: List[InputBox]) -> Dict[str,int]:
        res = {"erg":0}
        for box in boxes:
            res["erg"] += box.getValue()
            for token in box.getTokens():
                if token.getId().toString() not in res.keys():
                    res[token.getId().toString()] = token.getValue()
                else:
                    res[token.getId().toString()] += token.getValue()      
        return res

    def mapToErgoTokenList(map: Dict[str,int]) -> List[ErgoToken]:
        tts = []
        for entry in map:
            tts.append(ErgoToken(entry,map[entry]))
        return tts

    def getSigmaBooleanFromAddress(address: str) -> str:
        # we need a SigmaProp for ErgoAuth. Every address is a sigmaprop, so convert
        addressSigmaProp = SigmaProp.createFromAddress(Address.create(address))
        sigmaBoolean = base64.b64encode(addressSigmaProp.toBytes()).decode()
        return sigmaBoolean

    def verifyErgoAuthSignedMessage(address: str, message: str, signedMessage: str, proof: str) -> bool:
        addressSigmaProp = SigmaProp.createFromAddress(Address.create(address))
        proofBytes = base64.b64decode(proof)
        return ErgoAuthUtils.verifyResponse(addressSigmaProp, message, signedMessage, proofBytes)



import java.lang
import scala
import scala.collection
import scala.reflect
import scala.runtime
import scala.util
import typing



_Breaks__TryBlock__T = typing.TypeVar('_Breaks__TryBlock__T')  # <T>
class Breaks:
    def __init__(self): ...
    def breakable(self, op: scala.Function0[scala.runtime.BoxedUnit]) -> None: ...
    def scala$util$control$Breaks$$breakException(self) -> 'BreakControl': ...
    _tryBreakable__T = typing.TypeVar('_tryBreakable__T')  # <T>
    def tryBreakable(self, op: scala.Function0[_tryBreakable__T]) -> 'Breaks.TryBlock'[_tryBreakable__T]: ...
    class TryBlock(typing.Generic[_Breaks__TryBlock__T]):
        def catchBreak(self, onBreak: scala.Function0[_Breaks__TryBlock__T]) -> _Breaks__TryBlock__T: ...

class NoStackTrace:
    @staticmethod
    def $init$($this: 'NoStackTrace') -> None: ...
    def fillInStackTrace(self) -> java.lang.Throwable: ...
    @staticmethod
    def noSuppression() -> bool: ...

class NonFatal:
    @staticmethod
    def apply(t: java.lang.Throwable) -> bool: ...
    @staticmethod
    def unapply(t: java.lang.Throwable) -> scala.Option[java.lang.Throwable]: ...

class ControlThrowable(NoStackTrace): ...

class BreakControl(java.lang.Throwable, ControlThrowable):
    def __init__(self): ...
    def fillInStackTrace(self) -> java.lang.Throwable: ...

_Exception__By__T = typing.TypeVar('_Exception__By__T')  # <T>
_Exception__By__R = typing.TypeVar('_Exception__By__R')  # <R>
_Exception__Catch__T = typing.TypeVar('_Exception__Catch__T')  # <T>
class Exception:
    _allCatch__T = typing.TypeVar('_allCatch__T')  # <T>
    @staticmethod
    def allCatch() -> 'Exception.Catch'[_allCatch__T]: ...
    _allCatcher__T = typing.TypeVar('_allCatcher__T')  # <T>
    @staticmethod
    def allCatcher() -> scala.PartialFunction[java.lang.Throwable, _allCatcher__T]: ...
    _catching_0__T = typing.TypeVar('_catching_0__T')  # <T>
    _catching_1__T = typing.TypeVar('_catching_1__T')  # <T>
    @typing.overload
    @staticmethod
    def catching(c: scala.PartialFunction[java.lang.Throwable, _catching_0__T]) -> 'Exception.Catch'[_catching_0__T]: ...
    @typing.overload
    @staticmethod
    def catching(exceptions: scala.collection.Seq[typing.Type[typing.Any]]) -> 'Exception.Catch'[_catching_1__T]: ...
    _catchingPromiscuously_0__T = typing.TypeVar('_catchingPromiscuously_0__T')  # <T>
    _catchingPromiscuously_1__T = typing.TypeVar('_catchingPromiscuously_1__T')  # <T>
    @typing.overload
    @staticmethod
    def catchingPromiscuously(c: scala.PartialFunction[java.lang.Throwable, _catchingPromiscuously_0__T]) -> 'Exception.Catch'[_catchingPromiscuously_0__T]: ...
    @typing.overload
    @staticmethod
    def catchingPromiscuously(exceptions: scala.collection.Seq[typing.Type[typing.Any]]) -> 'Exception.Catch'[_catchingPromiscuously_1__T]: ...
    _failAsValue__T = typing.TypeVar('_failAsValue__T')  # <T>
    @staticmethod
    def failAsValue(exceptions: scala.collection.Seq[typing.Type[typing.Any]], value: scala.Function0[_failAsValue__T]) -> 'Exception.Catch'[_failAsValue__T]: ...
    _failing__T = typing.TypeVar('_failing__T')  # <T>
    @staticmethod
    def failing(exceptions: scala.collection.Seq[typing.Type[typing.Any]]) -> 'Exception.Catch'[scala.Option[_failing__T]]: ...
    _handling__T = typing.TypeVar('_handling__T')  # <T>
    @staticmethod
    def handling(exceptions: scala.collection.Seq[typing.Type[typing.Any]]) -> 'Exception.By'[scala.Function1[java.lang.Throwable, _handling__T], 'Exception.Catch'[_handling__T]]: ...
    @staticmethod
    def ignoring(exceptions: scala.collection.Seq[typing.Type[typing.Any]]) -> 'Exception.Catch'[scala.runtime.BoxedUnit]: ...
    _mkCatcher__Ex = typing.TypeVar('_mkCatcher__Ex', bound=java.lang.Throwable)  # <Ex>
    _mkCatcher__T = typing.TypeVar('_mkCatcher__T')  # <T>
    @staticmethod
    def mkCatcher(isDef: scala.Function1[_mkCatcher__Ex, typing.Any], f: scala.Function1[_mkCatcher__Ex, _mkCatcher__T], evidence$1: scala.reflect.ClassTag[_mkCatcher__Ex]) -> scala.PartialFunction[java.lang.Throwable, _mkCatcher__T]: ...
    _mkThrowableCatcher__T = typing.TypeVar('_mkThrowableCatcher__T')  # <T>
    @staticmethod
    def mkThrowableCatcher(isDef: scala.Function1[java.lang.Throwable, typing.Any], f: scala.Function1[java.lang.Throwable, _mkThrowableCatcher__T]) -> scala.PartialFunction[java.lang.Throwable, _mkThrowableCatcher__T]: ...
    @staticmethod
    def noCatch() -> 'Exception.Catch'[scala.runtime.Nothing.]: ...
    _nonFatalCatch__T = typing.TypeVar('_nonFatalCatch__T')  # <T>
    @staticmethod
    def nonFatalCatch() -> 'Exception.Catch'[_nonFatalCatch__T]: ...
    _nonFatalCatcher__T = typing.TypeVar('_nonFatalCatcher__T')  # <T>
    @staticmethod
    def nonFatalCatcher() -> scala.PartialFunction[java.lang.Throwable, _nonFatalCatcher__T]: ...
    @staticmethod
    def nothingCatcher() -> scala.PartialFunction[java.lang.Throwable, scala.runtime.Nothing.]: ...
    @staticmethod
    def shouldRethrow(x: java.lang.Throwable) -> bool: ...
    _throwableSubtypeToCatcher__Ex = typing.TypeVar('_throwableSubtypeToCatcher__Ex', bound=java.lang.Throwable)  # <Ex>
    _throwableSubtypeToCatcher__T = typing.TypeVar('_throwableSubtypeToCatcher__T')  # <T>
    @staticmethod
    def throwableSubtypeToCatcher(pf: scala.PartialFunction[_throwableSubtypeToCatcher__Ex, _throwableSubtypeToCatcher__T], evidence$2: scala.reflect.ClassTag[_throwableSubtypeToCatcher__Ex]) -> scala.PartialFunction[java.lang.Throwable, _throwableSubtypeToCatcher__T]: ...
    _ultimately__T = typing.TypeVar('_ultimately__T')  # <T>
    @staticmethod
    def ultimately(body: scala.Function0[scala.runtime.BoxedUnit]) -> 'Exception.Catch'[_ultimately__T]: ...
    _unwrapping__T = typing.TypeVar('_unwrapping__T')  # <T>
    @staticmethod
    def unwrapping(exceptions: scala.collection.Seq[typing.Type[typing.Any]]) -> 'Exception.Catch'[_unwrapping__T]: ...
    class By(typing.Generic[_Exception__By__T, _Exception__By__R]):
        def __init__(self, f: scala.Function1[_Exception__By__T, _Exception__By__R]): ...
        def by(self, x: _Exception__By__T) -> _Exception__By__R: ...
    class Catch(scala.util.control.Exception.Described, typing.Generic[_Exception__Catch__T]):
        def __init__(self, pf: scala.PartialFunction[java.lang.Throwable, _Exception__Catch__T], fin: scala.Option['Exception.Finally'], rethrow: scala.Function1[java.lang.Throwable, typing.Any]): ...
        def andFinally(self, body: scala.Function0[scala.runtime.BoxedUnit]) -> 'Exception.Catch'[_Exception__Catch__T]: ...
        _apply__U = typing.TypeVar('_apply__U')  # <U>
        def apply(self, body: scala.Function0[_apply__U]) -> _apply__U: ...
        def desc(self) -> str: ...
        _either__U = typing.TypeVar('_either__U')  # <U>
        def either(self, body: scala.Function0[_either__U]) -> scala.util.Either[java.lang.Throwable, _either__U]: ...
        def fin(self) -> scala.Option['Exception.Finally']: ...
        def name(self) -> str: ...
        _opt__U = typing.TypeVar('_opt__U')  # <U>
        def opt(self, body: scala.Function0[_opt__U]) -> scala.Option[_opt__U]: ...
        def pf(self) -> scala.PartialFunction[java.lang.Throwable, _Exception__Catch__T]: ...
        def rethrow(self) -> scala.Function1[java.lang.Throwable, typing.Any]: ...
        def scala$util$control$Exception$Described$$_desc(self) -> str: ...
        def scala$util$control$Exception$Described$$_desc_$eq(self, x$1: str) -> None: ...
        def toEither(self) -> 'Exception.Catch'[scala.util.Either[java.lang.Throwable, _Exception__Catch__T]]: ...
        def toOption(self) -> 'Exception.Catch'[scala.Option[_Exception__Catch__T]]: ...
        def toString(self) -> str: ...
        def toTry(self) -> 'Exception.Catch'[scala.util.Try[_Exception__Catch__T]]: ...
        _withApply__U = typing.TypeVar('_withApply__U')  # <U>
        def withApply(self, f: scala.Function1[java.lang.Throwable, _withApply__U]) -> 'Exception.Catch'[_withApply__U]: ...
        def withDesc(self, s: str) -> 'Exception.Described': ...
        _withTry__U = typing.TypeVar('_withTry__U')  # <U>
        def withTry(self, body: scala.Function0[_withTry__U]) -> scala.util.Try[_withTry__U]: ...
    class Catch$:
        MODULE$: typing.ClassVar['Exception.Catch.'] = ...
        def __init__(self): ...
        _$lessinit$greater$default$2__T = typing.TypeVar('_$lessinit$greater$default$2__T')  # <T>
        def $lessinit$greater$default$2(self) -> scala.Option['Exception.Finally']: ...
        _$lessinit$greater$default$3__T = typing.TypeVar('_$lessinit$greater$default$3__T')  # <T>
        def $lessinit$greater$default$3(self) -> scala.Function1[java.lang.Throwable, typing.Any]: ...
    class Described:
        @staticmethod
        def $init$($this: 'Exception.Described') -> None: ...
        def desc(self) -> str: ...
        def name(self) -> str: ...
        def scala$util$control$Exception$Described$$_desc(self) -> str: ...
        def scala$util$control$Exception$Described$$_desc_$eq(self, x$1: str) -> None: ...
        def toString(self) -> str: ...
        def withDesc(self, s: str) -> 'Exception.Described': ...
    class Finally(scala.util.control.Exception.Described):
        def __init__(self, body: scala.Function0[scala.runtime.BoxedUnit]): ...
        def desc(self) -> str: ...
        def invoke(self) -> None: ...
        def name(self) -> str: ...
        def scala$util$control$Exception$Described$$_desc(self) -> str: ...
        def scala$util$control$Exception$Described$$_desc_$eq(self, x$1: str) -> None: ...
        def toString(self) -> str: ...
        def withDesc(self, s: str) -> 'Exception.Described': ...

_TailCalls__Call__A = typing.TypeVar('_TailCalls__Call__A')  # <A>
_TailCalls__Cont__A = typing.TypeVar('_TailCalls__Cont__A')  # <A>
_TailCalls__Cont__B = typing.TypeVar('_TailCalls__Cont__B')  # <B>
_TailCalls__Done__A = typing.TypeVar('_TailCalls__Done__A')  # <A>
_TailCalls__TailRec__A = typing.TypeVar('_TailCalls__TailRec__A')  # <A>
class TailCalls:
    _done__A = typing.TypeVar('_done__A')  # <A>
    @staticmethod
    def done(result: _done__A) -> 'TailCalls.TailRec'[_done__A]: ...
    _tailcall__A = typing.TypeVar('_tailcall__A')  # <A>
    @staticmethod
    def tailcall(rest: scala.Function0['TailCalls.TailRec'[_tailcall__A]]) -> 'TailCalls.TailRec'[_tailcall__A]: ...
    class Call(scala.util.control.TailCalls.TailRec[_TailCalls__Call__A], scala.Product, scala.Serializable, typing.Generic[_TailCalls__Call__A]):
        def __init__(self, rest: scala.Function0['TailCalls.TailRec'[_TailCalls__Call__A]]): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        _copy__A = typing.TypeVar('_copy__A')  # <A>
        def copy(self, rest: scala.Function0['TailCalls.TailRec'[typing.Any]]) -> 'TailCalls.Call'[typing.Any]: ...
        _copy$default$1__A = typing.TypeVar('_copy$default$1__A')  # <A>
        def copy$default$1(self) -> scala.Function0['TailCalls.TailRec'[typing.Any]]: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> str: ...
        def rest(self) -> scala.Function0['TailCalls.TailRec'[_TailCalls__Call__A]]: ...
        def toString(self) -> str: ...
    class Call$(scala.Serializable):
        MODULE$: typing.ClassVar['TailCalls.Call.'] = ...
        def __init__(self): ...
        _apply__A = typing.TypeVar('_apply__A')  # <A>
        def apply(self, rest: scala.Function0['TailCalls.TailRec'[_apply__A]]) -> 'TailCalls.Call'[_apply__A]: ...
        def toString(self) -> str: ...
        _unapply__A = typing.TypeVar('_unapply__A')  # <A>
        def unapply(self, x$0: 'TailCalls.Call'[_unapply__A]) -> scala.Option[scala.Function0['TailCalls.TailRec'[_unapply__A]]]: ...
    class Cont(scala.util.control.TailCalls.TailRec[_TailCalls__Cont__B], scala.Product, scala.Serializable, typing.Generic[_TailCalls__Cont__A, _TailCalls__Cont__B]):
        def __init__(self, a: 'TailCalls.TailRec'[_TailCalls__Cont__A], f: scala.Function1[_TailCalls__Cont__A, 'TailCalls.TailRec'[_TailCalls__Cont__B]]): ...
        def a(self) -> 'TailCalls.TailRec'[_TailCalls__Cont__A]: ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        _copy__A = typing.TypeVar('_copy__A')  # <A>
        _copy__B = typing.TypeVar('_copy__B')  # <B>
        def copy(self, a: 'TailCalls.TailRec'[typing.Any], f: scala.Function1[typing.Any, 'TailCalls.TailRec'[typing.Any]]) -> 'TailCalls.Cont'[typing.Any, typing.Any]: ...
        _copy$default$1__A = typing.TypeVar('_copy$default$1__A')  # <A>
        _copy$default$1__B = typing.TypeVar('_copy$default$1__B')  # <B>
        def copy$default$1(self) -> 'TailCalls.TailRec'[typing.Any]: ...
        _copy$default$2__A = typing.TypeVar('_copy$default$2__A')  # <A>
        _copy$default$2__B = typing.TypeVar('_copy$default$2__B')  # <B>
        def copy$default$2(self) -> scala.Function1[typing.Any, 'TailCalls.TailRec'[typing.Any]]: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def f(self) -> scala.Function1[_TailCalls__Cont__A, 'TailCalls.TailRec'[_TailCalls__Cont__B]]: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> str: ...
        def toString(self) -> str: ...
    class Cont$(scala.Serializable):
        MODULE$: typing.ClassVar['TailCalls.Cont.'] = ...
        def __init__(self): ...
        _apply__A = typing.TypeVar('_apply__A')  # <A>
        _apply__B = typing.TypeVar('_apply__B')  # <B>
        def apply(self, a: 'TailCalls.TailRec'[_apply__A], f: scala.Function1[_apply__A, 'TailCalls.TailRec'[_apply__B]]) -> 'TailCalls.Cont'[_apply__A, _apply__B]: ...
        def toString(self) -> str: ...
        _unapply__A = typing.TypeVar('_unapply__A')  # <A>
        _unapply__B = typing.TypeVar('_unapply__B')  # <B>
        def unapply(self, x$0: 'TailCalls.Cont'[_unapply__A, _unapply__B]) -> scala.Option[scala.Tuple2['TailCalls.TailRec'[_unapply__A], scala.Function1[_unapply__A, 'TailCalls.TailRec'[_unapply__B]]]]: ...
    class Done(scala.util.control.TailCalls.TailRec[_TailCalls__Done__A], scala.Product, scala.Serializable, typing.Generic[_TailCalls__Done__A]):
        def __init__(self, value: _TailCalls__Done__A): ...
        def canEqual(self, x$1: typing.Any) -> bool: ...
        _copy__A = typing.TypeVar('_copy__A')  # <A>
        def copy(self, value: typing.Any) -> 'TailCalls.Done'[typing.Any]: ...
        _copy$default$1__A = typing.TypeVar('_copy$default$1__A')  # <A>
        def copy$default$1(self) -> typing.Any: ...
        def equals(self, x$1: typing.Any) -> bool: ...
        def hashCode(self) -> int: ...
        def productArity(self) -> int: ...
        def productElement(self, x$1: int) -> typing.Any: ...
        def productIterator(self) -> scala.collection.Iterator[typing.Any]: ...
        def productPrefix(self) -> str: ...
        def toString(self) -> str: ...
        def value(self) -> _TailCalls__Done__A: ...
    class Done$(scala.Serializable):
        MODULE$: typing.ClassVar['TailCalls.Done.'] = ...
        def __init__(self): ...
        _apply__A = typing.TypeVar('_apply__A')  # <A>
        def apply(self, value: _apply__A) -> 'TailCalls.Done'[_apply__A]: ...
        def toString(self) -> str: ...
        _unapply__A = typing.TypeVar('_unapply__A')  # <A>
        def unapply(self, x$0: 'TailCalls.Done'[_unapply__A]) -> scala.Option[_unapply__A]: ...
    class TailRec(typing.Generic[_TailCalls__TailRec__A]):
        def __init__(self): ...
        _flatMap__B = typing.TypeVar('_flatMap__B')  # <B>
        def flatMap(self, f: scala.Function1[_TailCalls__TailRec__A, 'TailCalls.TailRec'[_flatMap__B]]) -> 'TailCalls.TailRec'[_flatMap__B]: ...
        _map__B = typing.TypeVar('_map__B')  # <B>
        def map(self, f: scala.Function1[_TailCalls__TailRec__A, _map__B]) -> 'TailCalls.TailRec'[_map__B]: ...
        def result(self) -> _TailCalls__TailRec__A: ...
        def resume(self) -> scala.util.Either[scala.Function0['TailCalls.TailRec'[_TailCalls__TailRec__A]], _TailCalls__TailRec__A]: ...


class __module_protocol__(typing.Protocol):
    # A module protocol which reflects the result of ``jp.JPackage("scala.util.control")``.

    BreakControl: typing.Type[BreakControl]
    Breaks: typing.Type[Breaks]
    ControlThrowable: typing.Type[ControlThrowable]
    Exception: typing.Type[Exception]
    NoStackTrace: typing.Type[NoStackTrace]
    NonFatal: typing.Type[NonFatal]
    TailCalls: typing.Type[TailCalls]
